{*********************************************************}
{                                                         }
{    RIPC                                                 }
{    Remote IP Configuration Service                      }
{                                                         }
{    Copyright (c) 2005 Volodimir V. Flyonts              }
{    Send all comments to e-mail: flyonts@ukr.net         }
{                                                         }
{    Version 1.1 Beta                                     }
{                                                         }
{*********************************************************}

{

Copyright (C) 2005 Vladimir V. Flyonts

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, 
MA  02111-1307, USA.

Please send remarks, questions and bug reports to flyonts@ukr.net

}

unit RipSvc;

interface

uses
  Windows, WinSock, SysUtils, Md5, IpTypes, IpIfConst;

const

  RipDefPort = 1984;  

  { RIP commands }

  RIPC_VER = $00524556; // VER - get service & windows version
  RIPC_PAS = $00534150; // PAS - login using password
  RIPC_LSA = $0041534C; // LSA - list adapters (ascii)
  RIPC_LSB = $0042534C; // LSB - list adapters (binary)
  RIPC_IPC = $00435049; // IPC - ip configure (with md5)
  RIPC_DIS = $00534944; // DIS - disable adapter (by name)
  RIPC_ENA = $00414E45; // ENA - enable adapter (by name)
  RIPC_SDW = $00574453; // SDW - shutdown windows
  RIPC_RSW = $00575352; // RSW - restart windows
  RIPC_HLP = $00504C48; // HLP - help, list commands
  RIPC_QUIT= $54495551; // QUIT, no comments

  RIPC_VER1 = $43504952; // RIPC - RIPC/1.1 Windows XP ...
  RIPC_VER2 = $312E312F; // /1.1 ------^^^^

  { RIP ansver codes }

  RIPC_200 = $00303032;
  RIPC_220 = $00303232;
  RIPC_221 = $00313232;
  RIPC_230 = $00303332;
  RIPC_530 = $00303335;

  { RIP ascii ansvers }

  RIPA_125 = '125-Data connection already open; transfer starting.';
  
  RIPA_200 = '200-Command okay.';

  RIPA_220 = '220-Service ready for new user.';
  RIPA_221 = '221-Service closing control connection.';
  RIPA_225 = '225-Data connection open; no transfer in progress.';
  RIPA_226 = '226-Closing data connection.';
  RIPA_230 = '230-User logged in, proceed.';

  RIPA_421 = '421-Service not available, closing control connection.';
  RIPA_425 = '425-Can not open data connection.';
  RIPA_426 = '426-Connection closed; transfer aborted.';
  RIPA_451 = '451-Requested action aborted: local error in processing.';
  RIPA_452 = '452-Requested action not taken.';

  RIPA_500 = '500-Syntax error, command unrecognized.';
  RIPA_501 = '501-Syntax error in parameters or arguments.';
  RIPA_530 = '530-Not logged in.';
  RIPA_550 = '550-Access denied.';

  { IpTypes }

  MAX_IP_ADDR_COUNT           = 8;
  MAX_ADAPTER_COUNT           = 8;

type

  { TIPAddr }

  PIPAddr = ^TIPAddr;
  TIPAddr = packed record
    s_b1: byte;
    s_b2: byte;
    s_b3: byte;
    s_b4: byte;
  end;

  { TSession }

  PSession = ^TSession;
  TSession = record
    Active: Boolean;
    Socket: TSOCKET;
    SockAddr: TSockAddr;
    Passwd: TMD5Digest;
  end;

  { TIpAddrList }

  PIpAddrList = ^TIpAddrList;
  TIpAddrList = packed record
    Count: Cardinal;
    Adrs: packed array[0..MAX_IP_ADDR_COUNT - 1]of TIpAddrString; 
  end;

  { TNetworkParams }

  PRipFixedInfo = ^TRipFixedInfo;
  TRipFixedInfo = packed record
    HostName: packed array [0..MAX_HOSTNAME_LEN + 3] of Char;
    DomainName: packed array[0..MAX_DOMAIN_NAME_LEN + 3] of Char;
    DnsServerList: TIpAddrList;
    NodeType: UINT;
    ScopeId: array [0..MAX_SCOPE_ID_LEN + 3] of Char;
    EnableRouting: UINT;
    EnableProxy: UINT;
    EnableDns: UINT;
  end;

  { TRipAnsverHeader }

  PRipInfoHeader = ^TRipInfoHeader;
  TRipInfoHeader = packed record
    Status: Cardinal;
    TotalSize: Cardinal;
    HeaderSize: Cardinal;
    FixedSize: Cardinal;
    AdapterSize: Cardinal;
    AdapterCount: Cardinal;
    Sign: TMD5Digest;
  end;

  { TRipAdapterInfo }

  PRipAdapterInfo = ^TRipAdapterInfo;
  TRipAdapterInfo = packed record
    // TIpAdapterInfo
    Next: Pointer;
    ComboIndex: DWORD;
    AdapterName: array [0..MAX_ADAPTER_NAME_LENGTH + 3] of Char;
    Description: array [0..MAX_ADAPTER_DESCRIPTION_LENGTH + 3] of Char;
    AddressLength: UINT;
    Address: array [0..MAX_ADAPTER_ADDRESS_LENGTH - 1] of BYTE;
    Index: DWORD;
    Type_: UINT;
    DhcpEnabled: UINT;
    CurrentIpAddress: Pointer;
    IpAddressList: TIpAddrList;
    GatewayList: TIpAddrList;
    DhcpServer: TIpAddrList;
    HaveWins: BOOL;
    PrimaryWinsServer: TIpAddrList;
    SecondaryWinsServer: TIpAddrList;
    LeaseObtained: time_t;
    LeaseExpires: time_t;
    // TIpPerAdapterInfo
    AutoconfigEnabled: UINT;
    AutoconfigActive: UINT;
    CurrentDnsServer: Pointer;
    DnsServerList: TIpAddrList;
  end;

  { TAdaptersList }

  PAdapterList = ^TAdapterList;
  TAdapterList = packed array[0..MAX_ADAPTER_COUNT - 1]of TRipAdapterInfo;

  { TRipNetworkInfo }

  PRipNetworkInfo = ^TRipNetworkInfo;
  TRipNetworkInfo = packed record
    Header: TRipInfoHeader;
    FixedInfo: TRipFixedInfo;
    Adapters: TAdapterList;
  end;
  PRNI = PRipNetworkInfo;

  PRipIpConfigure = ^TRipIpConfigure;
  TRipIpConfigure = packed record
    Header: TRipInfoHeader;
    Adapter: TRipAdapterInfo;
  end;


function YesNo(Q: Cardinal): string;

function IpListToStr(List: PIpAddrList; Mask: Boolean = False): string;
function NodeTypeToStr(NodeType: Cardinal): string;
function AdapterTypeToStr(T: Cardinal): string;
function PhysAddrToStr(Address: array of Byte; AddressLength: Cardinal): string;

function IpListToSz(List: PIpAddrList; Sz: PChar; Mask: Boolean = False): Cardinal;
function IpListToMultiSz(List: PIpAddrList; Sz: PChar; Mask: Boolean = False): Cardinal;

implementation

function YesNo(Q: Cardinal): string;
begin
  if Q = 0 then
    Result := 'No'
  else
    Result := 'Yes'
end;

function IpListToStr(List: PIpAddrList; Mask: Boolean): string;
var
  I: Integer;
  S: string;
begin
  Result := '';
  if List = nil then
    Exit;
  for I := 0 to List^.Count - 1 do
  begin
    if Mask then
      S := string(List^.Adrs[I].IpMask.S)
    else
      S := string(List^.Adrs[I].IpAddress.S);
    if S <> '' then
    begin
      if Result <> '' then
        Result := Result + ', ';
      Result := Result + S;
    end;
  end;
end;

function IpListToSz(List: PIpAddrList; Sz: PChar; Mask: Boolean): Cardinal;
var
  I: Integer;
  S: PChar;
begin
  Sz^ := #0;
  for I := 0 to List.Count - 1 do
  begin
    if Sz^ <> #0 then
      StrCat(Sz, ',');
    if Mask then
      S := @List.Adrs[I].IpMask.S
    else
      S := @List.Adrs[I].IpAddress.S;
    StrCat(Sz, S);
  end;
  Result := StrLen(Sz);
end;

function IpListToMultiSz(List: PIpAddrList; Sz: PChar; Mask: Boolean): Cardinal;
var
  I,L: Integer;
  S: PChar;
begin
  Result := 0;
  for I := 0 to List.Count - 1 do
  begin
    if Mask then
      S := @List.Adrs[I].IpMask.S
    else
      S := @List.Adrs[I].IpAddress.S;
    L := StrLen(S) + 1;
    Move(S^, Sz^, L);
    Inc(Result, L);
    Inc(Sz, L);
  end;
  Inc(Result);
  Sz^ := #0;
end;

function NodeTypeToStr(NodeType: Cardinal): string;
begin
  case NodeType of
  BROADCAST_NODETYPE:
    Result := 'Broadcast';
  PEER_TO_PEER_NODETYPE:
    Result := 'PeerToPeer';
  MIXED_NODETYPE:
    Result := 'Mixed';
  HYBRID_NODETYPE:
    Result := 'Hybrid';
  else
    Result := 'Unknown';
  end;
end;

function AdapterTypeToStr(T: Cardinal): string;
begin
  case T of
  MIB_IF_TYPE_OTHER:
    Result := 'Other';
  MIB_IF_TYPE_ETHERNET:
    Result := 'Ethernet';
  MIB_IF_TYPE_TOKENRING:
    Result := 'Token Ring';
  MIB_IF_TYPE_FDDI:
    Result := 'FDDI';
  MIB_IF_TYPE_PPP:
    Result := 'PPP';
  MIB_IF_TYPE_LOOPBACK:
    Result := 'Loopback';
  MIB_IF_TYPE_SLIP:
    Result := 'Slip';
  else
    Result := 'Unknown';
  end;
end;

function PhysAddrToStr(Address: array of Byte; AddressLength: Cardinal): string;
var
  I: Integer;
begin
  Result := '';
  for I := 0 to AddressLength - 1 do
  begin
    if Result <> '' then Result := Result + '-';
    Result := Result + IntToHex(Address[I], 2);
  end;
end;


end.
