{*********************************************************}
{                                                         }
{    RIPC                                                 }
{    IP Config Form Unit                                  }
{                                                         }
{    Copyright (c) 2005 Volodimir V. Flyonts              }
{    Send all comments to e-mail: flyonts@ukr.net         }
{                                                         }
{    Version 1.1 Beta                                     }
{                                                         }
{*********************************************************}

{

Copyright (C) 2005 Vladimir V. Flyonts

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston,
MA  02111-1307, USA.

Please send remarks, questions and bug reports to flyonts@ukr.net

}

unit IpConfigForm;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, StdCtrls, Mask, IPAddress, ExtCtrls, RipSvc;

type
  TfrmIPConfig = class(TForm)
    btnOk: TButton;
    btnCancel: TButton;
    PageControl1: TPageControl;
    TabSheet1: TTabSheet;
    StaticText: TStaticText;
    btnRefresh: TButton;
    rbnEnableDHSP: TRadioButton;
    RadioButton1: TRadioButton;
    GroupBox3: TGroupBox;
    StaticText1: TStaticText;
    IPAddress1: TIPAddress;
    StaticText2: TStaticText;
    IPAddress2: TIPAddress;
    StaticText3: TStaticText;
    IPAddress3: TIPAddress;
    Panel1: TPanel;
    rbnEnableAutoDNS: TRadioButton;
    RadioButton2: TRadioButton;
    GroupBox2: TGroupBox;
    StaticText5: TStaticText;
    IPAddress4: TIPAddress;
    StaticText6: TStaticText;
    IPAddress5: TIPAddress;
    procedure RadioButton1Click(Sender: TObject);
    procedure RadioButton2Click(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure btnRefreshClick(Sender: TObject);
  private
    AdapterInfo: TRipAdapterInfo;
    { Private declarations }
    procedure UpdateAdapterInfo(GetInfoFlag: Boolean);
  public
    { Public declarations }
    procedure GetAdapterInfo(PInfo: PRipAdapterInfo);
    procedure SetAdapterInfo(PInfo: PRipAdapterInfo);
  end;

function IpConfigure(Info: PRipAdapterInfo): Boolean;

var
  frmIPConfig: TfrmIPConfig;

implementation

uses IpTypes;

{$R *.dfm}

function IpConfigure(Info: PRipAdapterInfo): Boolean;
begin
  Result := False;
  if Info = nil then
    Exit;
  with TfrmIPConfig.Create(Application) do
  try
    SetAdapterInfo(Info);
    Result := ShowModal = mrOk;
    if Result then
      GetAdapterInfo(Info);
  finally
    Free;
  end;
end;

procedure EnableIPControl(AIPControl: TIPAddress; AEnable: Boolean);
begin
  AIPControl.Enabled := AEnable;
  if AEnable then
    AIPControl.Color := clWindow
  else
    AIPControl.Color := clBtnFace
end;

procedure TfrmIPConfig.RadioButton1Click(Sender: TObject);
var
  EnableIP: Boolean;
begin
  EnableIP := not rbnEnableDHSP.Checked;
  // DHSP
  StaticText1.Enabled := EnableIP;
  EnableIPControl(IPAddress1, EnableIP);
  StaticText2.Enabled := EnableIP;
  EnableIPControl(IPAddress2, EnableIP);
  StaticText3.Enabled := EnableIP;
  EnableIPControl(IPAddress3, EnableIP);
  // DNS
  rbnEnableAutoDNS.Enabled := rbnEnableDHSP.Checked; 
  if not rbnEnableAutoDNS.Enabled then
    RadioButton2.Checked := True;
  if Assigned(RadioButton2.OnClick) then
    RadioButton2.OnClick(RadioButton2);
end;

procedure TfrmIPConfig.RadioButton2Click(Sender: TObject);
var
  EnableIP: Boolean;
begin
  EnableIP := not rbnEnableAutoDNS.Checked;
  EnableIPControl(IPAddress4, EnableIP);
  StaticText5.Enabled := EnableIP;
  EnableIPControl(IPAddress5, EnableIP);
  StaticText6.Enabled := EnableIP;
end;

procedure TfrmIPConfig.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = 27 then btnCancel.Click;
end;

procedure TfrmIPConfig.GetAdapterInfo(PInfo: PRipAdapterInfo);
begin
  if PInfo = nil then
    Exit;
  UpdateAdapterInfo(True);
  Move(AdapterInfo, PInfo^, SizeOf(TRipAdapterInfo));
end;

procedure TfrmIPConfig.SetAdapterInfo(PInfo: PRipAdapterInfo);
begin
  if PInfo = nil then
    Exit;
  Move(PInfo^, AdapterInfo, SizeOf(TRipAdapterInfo));
  UpdateAdapterInfo(False);
end;

procedure TfrmIPConfig.UpdateAdapterInfo(GetInfoFlag: Boolean);
const
  Zip = '0.0.0.0';
  Msk = '255.255.255.0';
begin
  if GetInfoFlag then
  begin
    // IP Address
    if rbnEnableDHSP.Checked then
    begin
      AdapterInfo.DhcpEnabled := 1;
      with AdapterInfo.IpAddressList.Adrs[0] do
      begin
        StrCopy(IpAddress.S, Zip);
        StrCopy(IpMask.S, Zip);
      end;
      with AdapterInfo.GatewayList.Adrs[0] do
      begin
        StrCopy(IpAddress.S, Zip);
        StrCopy(IpMask.S, Zip);
      end;
    end
    else // if rbnEnableDHSP.Checked 
    begin
      AdapterInfo.DhcpEnabled := 0;
      with AdapterInfo.IpAddressList.Adrs[0] do
      begin
        StrCopy(IpAddress.S, PChar(IPAddress1.Text));
        StrCopy(IpMask.S, PChar(IPAddress2.Text));
      end;
      with AdapterInfo.GatewayList.Adrs[0] do
      begin
        StrCopy(IpAddress.S, PChar(IPAddress3.Text));
      end;
    end; // if rbnEnableDHSP.Checked
    // DNS Server
    if rbnEnableAutoDNS.Checked then
    begin
      AdapterInfo.DnsServerList.Count := 0;
    end
    else
    begin
      AdapterInfo.DnsServerList.Count := 0;
      if not IPAddress4.IsBlank then
      begin
        AdapterInfo.DnsServerList.Count := 1;
        with AdapterInfo.DnsServerList.Adrs[0] do
          StrCopy(IpAddress.S, PChar(IPAddress4.Text));
      end;
      if (not IPAddress5.IsBlank) and (IPAddress5.Text <> ZIp) then
      begin
        AdapterInfo.DnsServerList.Count := 2;
        with AdapterInfo.DnsServerList.Adrs[1] do
          StrCopy(IpAddress.S, PChar(IPAddress4.Text));
      end;
    end;
  end
  else // if GetInfoFlag
  begin
    if AdapterInfo.DhcpEnabled = 0 then
    begin
      RadioButton1.Checked := True;
      with AdapterInfo.IpAddressList.Adrs[0] do
      begin
        IPAddress1.Text := string(IpAddress.S);
        IPAddress2.Text := string(IpMask.S);
      end;
      with AdapterInfo.GatewayList.Adrs[0] do
      begin
        IPAddress3.Text := string(IpAddress.S);
      end;
    end else
      rbnEnableDHSP.Checked := True;
    if (AdapterInfo.DnsServerList.Count > 0) and 
       (StrComp(AdapterInfo.DnsServerList.Adrs[0].IpAddress.S, ZIp) <> 0) then
    begin
      RadioButton2.Checked := True;
      with AdapterInfo.DnsServerList.Adrs[0] do
      begin
        IpAddress4.Text := string(IpAddress.S);
      end;
      if AdapterInfo.DnsServerList.Count > 1 then
        with AdapterInfo.DnsServerList.Adrs[1] do
        begin
          IpAddress5.Text := string(IpAddress.S);
        end;
    end else
      rbnEnableAutoDNS.Checked := rbnEnableDHSP.Checked;
    RadioButton1Click(RadioButton1);
    RadioButton2Click(RadioButton2);
  end; // if GetInfoFlag
end;

procedure TfrmIPConfig.btnRefreshClick(Sender: TObject);
begin
  UpdateAdapterInfo(False);
end;

end.
