{*********************************************************}
{                                                         }
{    RIPC                                                 }
{    Main Form Unit                                       }
{                                                         }
{    Copyright (c) 2005 Volodimir V. Flyonts              }
{    Send all comments to e-mail: flyonts@ukr.net         }
{                                                         }
{    Version 1.1 Beta 2                                   }
{                                                         }
{*********************************************************}

{

Copyright (C) 2005 Vladimir V. Flyonts

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston,
MA  02111-1307, USA.

Please send remarks, questions and bug reports to flyonts@ukr.net

}

unit MainForm;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, Tabs, ImgList, Menus, ExtCtrls, MD5, RipUtils, ActnList;

type
  TfrmMain = class(TForm)
    Bottom1: TMenuItem;
    ImageList1: TImageList;
    Left1: TMenuItem;
    lsvAdapters: TListView;
    lsvComputers: TListView;
    PopupMenu1: TPopupMenu;
    PopupMenu2: TPopupMenu;
    PopupMenu3: TPopupMenu;
    Right1: TMenuItem;
    pgcMain: TPageControl;
    tbsAdapters: TTabSheet;
    tbsComputers: TTabSheet;
    Top1: TMenuItem;
    ActionList1: TActionList;
    ComputerAddHostAction: TAction;
    AdapterPropertiesAction: TAction;
    ComputerDeleteHostAction: TAction;
    ComputerPropertiesAction: TAction;
    AddHost1: TMenuItem;
    DeleteHost1: TMenuItem;
    Properties1: TMenuItem;
    RefreshAction: TAction;
    Refresh1: TMenuItem;
    Properties2: TMenuItem;
    AdapterDisableAction: TAction;
    AdapterEnableAction: TAction;
    ComputerPingAction: TAction;
    PingHost1: TMenuItem;
    DisableAdapter1: TMenuItem;
    EnableAdapter1: TMenuItem;
    Refresh2: TMenuItem;
    ComputerShutdownAction: TAction;
    ComputerRestartAction: TAction;
    RestartWindows1: TMenuItem;
    ShutdownWinodws1: TMenuItem;
    pgcHistory: TTabSheet;
    lsvHistory: TListView;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Bottom1Click(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure ComputerAddHostActionExecute(Sender: TObject);
    procedure ComputerPropertiesActionExecute(Sender: TObject);
    procedure ComputerDeleteHostActionExecute(Sender: TObject);
    procedure ComputerDeleteHostActionUpdate(Sender: TObject);
    procedure ComputerPropertiesActionUpdate(Sender: TObject);
    procedure lsvComputersDblClick(Sender: TObject);
    procedure RefreshActionExecute(Sender: TObject);
    procedure lsvAdaptersDblClick(Sender: TObject);
    procedure AdapterPropertiesActionUpdate(Sender: TObject);
    procedure AdapterPropertiesActionExecute(Sender: TObject);
    procedure ComputerPingActionExecute(Sender: TObject);
    procedure ComputerPingActionUpdate(Sender: TObject);
    procedure AdapterDisableActionExecute(Sender: TObject);
    procedure AdapterDisableActionUpdate(Sender: TObject);
    procedure AdapterEnableActionExecute(Sender: TObject);
    procedure AdapterEnableActionUpdate(Sender: TObject);
    procedure ComputerShutdownActionExecute(Sender: TObject);
    procedure ComputerShutdownActionUpdate(Sender: TObject);
    procedure ComputerRestartActionExecute(Sender: TObject);
    procedure ComputerRestartActionUpdate(Sender: TObject);
  private
    Logined: Boolean;
    Changed: Boolean;
    ActiveUser: string;
    ActivePwd: TMD5Digest;
    { Private declarations }
    function Login: Boolean;
    function LoginAsUser(const UserName, Password: string): Boolean;
    function ReadUserConfig(const Buffer; Size: Cardinal): Boolean;
    procedure ChangePassword;
    procedure RefreshComputers;
    procedure SaveActiveUserConfig;
    procedure ShowAboutBox;
    procedure SaveUserConfig(var Buffer; var Size: Cardinal);
    procedure RegisterComputer(Info: TComputerInfo);
    procedure OnGetAdapterItem(Sender: TObject; var Item: TListItem);
    procedure OnLogMessageString(Sender: TObject; Kind: TMessageKind; Msg: string);
    procedure OnWmSysCommand(var Msg: TWMSysCommand); message WM_SysCommand;
  public
    { Public declarations }
  end;

  procedure CryptBuffer(const Source; var Dest; Size: Cardinal;
    const Key; KeySize: Cardinal);
  procedure DecryptBuffer(const Source; var Dest; Size: Cardinal;
    const Key; KeySize: Cardinal);

var
  frmMain: TfrmMain;

implementation

uses
  LoginForm, PcConfigForm, IpConfigForm, RipSvc;

const

  EOL = #13#10;

  MI_CHANGEPASSWORD = $1984;
  MI_ABOUTBOX = $1988;

  RegKeyRIP = 'Software\RemoteIP';
  RegKeyForm = RegKeyRIP + '\Form';
  RegKeyUser = RegKeyRIP + '\User';

  RegValLeft = 'Left';
  RegValTop = 'Top';
  RegValWidth = 'Width';
  RegValHeight = 'Height';
  RegValTabs = 'Tabs';
  RegValUser = 'User'; 

resourcestring

  SWrongPassword = 'The password is incorrect. Please retype your password.';
  SPasswordChanged = 'Your password has been changed.';
  SCannotOpenUserConfig = 'Can not open users configuration registry key!';
  SCannotConfigureIp = 'Can not configure remote adapter!';
  SCannotExecuteCommand = 'Can not execute command!';
  SUserConfigNotFound = 'User with name "%s" not found, create new one?';
  SChangePassword = '&Change Password...';
  SAboutProgram = '&About...';

  SCommonConfirm = 'Are you sure you want to %s "%s"?';
  SDeleteComputer = 'delete';
  SEnableAdapter = 'enable';
  SDisableAdapter = 'disable';
  SShutdownComputer = 'shutdown';
  SRestartComputer = 'restart';

  SAboutText = 'Remote IP Config Utility' + EOL +
    ' ' + EOL +
    'Version 1.1.2.220 - 01.06.2005' + EOL +
    ' ' + EOL +
    'Copyright  Vladimir V. Flyonts' + EOL +
    'E-mail: flyonts@ukr.net';                            

{$R *.dfm}

procedure FreeListItemsData(List: TListView);
var
  I: Integer;
begin
  if Assigned(List) then
    for I := 0 to List.Items.Count - 1 do
      with List.Items[I] do
        if Assigned(Data) then
        begin
          TObject(Data).Free;
          Data := nil;
        end;
end;

function TfrmMain.Login: Boolean;
var
  UserName, Password: string;
begin
  Result := False;
  UserName := ActiveUser;
  Password := '';
  while not Result and LoginForm.Login(UserName, Password) do
  begin
    Result := LoginAsUser(UserName, Password);
    if not Result then
    begin
      Sleep(500);
      MessageBox(Handle, PChar(SWrongPassword), PChar(Caption), MB_ICONWARNING + MB_OK);
      Password := '';
    end;
  end;
end;

function TfrmMain.LoginAsUser(const UserName, Password: string): Boolean;
var
  RegKey: HKEY;
  ErrCode: Longint;
  Buffer: Pointer;
  BfSize: Cardinal;
  MD5Pwd: TMD5Digest;
begin
  Result := False;
  if RegCreateKeyEx(HKEY_CURRENT_USER, RegKeyUser,
    0, nil, 0, KEY_READ, nil, RegKey, nil) = ERROR_SUCCESS then
  begin
    ErrCode := RegQueryValueEx(RegKey, PChar(UserName), nil, nil, nil, @BfSize);
    if ErrCode = ERROR_SUCCESS then
    begin
      Buffer := AllocMem(BfSize);
      if RegQueryValueEx(RegKey, PChar(UserName),
        nil, nil, Buffer, @BfSize) = ERROR_SUCCESS then
      begin
        MD5Pwd := MD5String(Password);
        DecryptBuffer(Buffer^, Buffer^, BfSize, MD5Pwd, SizeOf(MD5Pwd));
        Result := ReadUserConfig(Buffer^, BfSize);
        // if login ok set active user & password
        if Result then
        begin
          ActiveUser := UserName;
          ActivePwd := MD5Pwd;
        end;
      end;
      FreeMem(Buffer);
    end else
    if ErrCode = ERROR_FILE_NOT_FOUND then
    begin
      Result := MessageBox(Handle,
        PChar(Format(SUserConfigNotFound, [UserName])),
        PChar(Caption), MB_ICONQUESTION + MB_YESNO) = IDYES;
      if Result then
      begin
        ActiveUser := UserName;
        ActivePwd := MD5String('');
        Changed := True;
      end;
    end;
    RegCloseKey(RegKey);
  end else
    MessageBox(Handle, PChar(SCannotOpenUserConfig), PChar(Caption), MB_ICONERROR + MB_OK);
end;

procedure TfrmMain.ShowAboutBox;
begin
  MessageBox(Handle, PChar(SAboutText), PChar(Caption), MB_OK);
end;

procedure TfrmMain.ChangePassword;
var
  UserName,
  OldPassword,
  NewPassword,
  ConfirmPassword: string;
begin
  UserName := ActiveUser;
  OldPassword := '';
  NewPassword := '';
  ConfirmPassword := '';
  while LoginForm.ChangePassword(UserName,
    OldPassword, NewPassword, ConfirmPassword) do
  begin
    if AnsiSameText(ActiveUser, UserName) and
       AnsiSameStr(NewPassword, ConfirmPassword) and
       MD5DigestCompare(ActivePwd, MD5String(OldPassword)) then
    begin
      ActivePwd := MD5String(NewPassword);
      SaveActiveUserConfig;
      MessageBox(Handle, PChar(SPasswordChanged), PChar(Caption), MB_ICONINFORMATION + MB_OK);
      Break;
    end else
      MessageBox(Handle, PChar(SWrongPassword), PChar(Caption), MB_ICONWARNING + MB_OK);
    Sleep(500);
  end;
end;

procedure TfrmMain.SaveActiveUserConfig;
const
  MaxPcInfoSize = 2 + 1 + 1 + 255 + 255;
var
  RegKey: HKEY;
  Buffer: Pointer;
  BfSize: Cardinal;
begin
  if RegCreateKeyEx(HKEY_CURRENT_USER, RegKeyUser,
    0, nil, 0, KEY_WRITE, nil, RegKey, nil) = ERROR_SUCCESS then
  begin
    BfSize := SizeOf(TMD5Digest) + MaxPcInfoSize * lsvComputers.Items.Count;
    Buffer := AllocMem(BfSize);
    SaveUserConfig(Buffer^, BfSize);
    CryptBuffer(Buffer^, Buffer^, BfSize, ActivePwd, SizeOf(ActivePwd));
    RegSetValueEx(RegKey, PChar(ActiveUser), 0, REG_BINARY, Buffer, BfSize);
    RegCloseKey(RegKey);
  end;
end;

procedure TfrmMain.FormCreate(Sender: TObject);
var
  K: HKEY;
  T: Integer;
  S: Cardinal;
  U: array[0..MaxByte]of Char;
  SysMenu : THandle;
begin
  { Add "Change Password" }
  SysMenu := GetSystemMenu(Handle, False);
  AppendMenu(SysMenu, MF_SEPARATOR, 0, #0);             
  AppendMenu(SysMenu, MF_BYPOSITION, MI_CHANGEPASSWORD, PChar(SChangePassword));
  AppendMenu(SysMenu, MF_BYPOSITION, MI_ABOUTBOX, PChar(SAboutProgram));
  { Init Defaults }
  Changed := False;
  Logined := False;
  ActiveUser := 'na';
  Application.Title := Caption;
  { Load Settings }
  if RegCreateKeyEx(HKEY_CURRENT_USER, RegKeyForm,
    0, nil, 0, KEY_READ, nil, K, nil) = ERROR_SUCCESS then
  begin
    S := SizeOf(T);
    if RegQueryValueEx(K, RegValLeft, nil, nil, @T, @S) = ERROR_SUCCESS then
      if (T >= 0) and (T < GetSystemMetrics(SM_CXSCREEN)) then Left := T;
    S := SizeOf(T);
    if RegQueryValueEx(K, RegValTop, nil, nil, @T, @S) = ERROR_SUCCESS then
      if (T >= 0) and (T < GetSystemMetrics(SM_CYSCREEN)) then Top := T;
    S := SizeOf(T);
    if RegQueryValueEx(K, RegValWidth, nil, nil, @T, @S) = ERROR_SUCCESS then
      Width := T;
    S := SizeOf(T);
    if RegQueryValueEx(K, RegValHeight, nil, nil, @T, @S) = ERROR_SUCCESS then
      Height := T;
    S := SizeOf(T);
    if RegQueryValueEx(K, RegValTabs, nil, nil, @T, @S) = ERROR_SUCCESS then
      if (T >= 0) and (T < 4) then pgcMain.TabPosition := TTabPosition(T);
    S := SizeOf(U);
    if RegQueryValueEx(K, RegValUser, nil, nil, @U, @S) = ERROR_SUCCESS then
      SetString(ActiveUser, U, S);
    RegCloseKey(K);
  end;
end;

procedure TfrmMain.FormDestroy(Sender: TObject);
var
  K: HKEY;
  T: Integer;
begin
  { Save Settings }
  if RegCreateKeyEx(HKEY_CURRENT_USER, RegKeyForm,
    0, nil, 0, KEY_WRITE, nil, K, nil) = ERROR_SUCCESS then
  begin
    RegSetValueEx(K, RegValLeft, 0, REG_DWORD, @Left, SizeOf(Left));
    RegSetValueEx(K, RegValTop, 0, REG_DWORD, @Top, SizeOf(Top));
    RegSetValueEx(K, RegValWidth, 0, REG_DWORD, @Width, SizeOf(Width));
    RegSetValueEx(K, RegValHeight, 0, REG_DWORD, @Height, SizeOf(Height));
    T := Integer(pgcMain.TabPosition);
    RegSetValueEx(K, RegValTabs, 0, REG_DWORD, @T, SizeOf(T));
    RegSetValueEx(K, RegValUser, 0, REG_SZ, PChar(ActiveUser),
      Length(ActiveUser));
    RegCloseKey(K);
  end;
  { Save active user config }
  if Changed then
    SaveActiveUserConfig;
  { Free Memory }
  FreeListItemsData(lsvComputers);
  FreeListItemsData(lsvAdapters);
end;

procedure TfrmMain.OnWmSysCommand(var Msg: TWMSysCommand);
begin
  inherited;
  case Msg.CmdType of
  MI_CHANGEPASSWORD: ChangePassword;
  MI_ABOUTBOX: ShowAboutBox;
  end;
end;

procedure TfrmMain.Bottom1Click(Sender: TObject);
begin
  pgcMain.TabPosition := TTabPosition((Sender as TMenuItem).Tag)
end;

procedure TfrmMain.FormActivate(Sender: TObject);
begin
  if (not Logined) then
  begin
    Logined := Login;
    if Logined then
      RefreshComputers
    else
      Close;
  end;
end;

function rol(a,c: Byte): Byte;
asm
  push ecx
  mov cl,dl
  rol al,cl
  pop ecx
end;

function ror(a,c: Byte): Byte;
asm
  push ecx
  mov cl,dl
  ror al,cl
  pop ecx
end;

procedure CryptBuffer(const Source; var Dest; Size: Cardinal;
  const Key; KeySize: Cardinal);
var
  I,J: Cardinal;
  B,K: Byte;
begin
  J := 0;
  for I := 0 to Size - 1 do
  begin
    B := TByteArray(Source)[I];
    B := B xor TByteArray(Key)[J];
    if J+1 >= KeySize then
      K := TByteArray(Key)[0]
    else
      K := TByteArray(Key)[J+1];
    B := rol(B, K);
    TByteArray(Dest)[I] := B;
    Inc(J);
    if J >= KeySize then J := 0;
  end;
end;

procedure DecryptBuffer(const Source; var Dest; Size: Cardinal;
  const Key; KeySize: Cardinal);
var
  I,J: Cardinal;
  B,K: Byte;
begin
  J := 0;
  for I := 0 to Size - 1 do
  begin
    B := TByteArray(Source)[I];
    if J+1 >= KeySize then
      K := TByteArray(Key)[0]
    else
      K := TByteArray(Key)[J+1];
    B := ror(B, K);
    B := B xor TByteArray(Key)[J];
    TByteArray(Dest)[I] := B;
    Inc(J);
    if J >= KeySize then J := 0;
  end;
end;

function TfrmMain.ReadUserConfig(const Buffer; Size: Cardinal): Boolean;
var
  Offset: Cardinal;
  Sign: TMD5Digest;
  Info: TComputerInfo;
begin
  Offset := SizeOf(TMD5Digest);
  Sign := MD5Buffer(TByteArray(Buffer)[Offset], Size-Offset);
  Result := MD5DigestCompare(TMD5Digest(Buffer), Sign);
  if not Result then
    Exit;
  while Offset < Size do
  begin
    Info := TComputerInfo.Create('', '', RipDefPort);
    Inc(Offset, Info.Read(TByteArray(Buffer)[Offset]));
    RegisterComputer(Info);
  end;
end;

procedure TfrmMain.SaveUserConfig(var Buffer; var Size: Cardinal);
var
  I: Integer;
  Offset: Cardinal;
  Info: TComputerInfo;
begin
  Offset := SizeOf(TMD5Digest);
  for I := 0 to lsvComputers.Items.Count - 1 do
  begin
    if Offset > Size then Break;
    Info := TComputerInfo(lsvComputers.Items[I].Data);
    if Assigned(Info) then
      Inc(Offset, Info.Write(TByteArray(Buffer)[Offset]));
  end;
  Size := Offset;
  Offset := SizeOf(TMD5Digest);
  TMD5Digest(Buffer) := MD5Buffer(TByteArray(Buffer)[Offset], Size-Offset);
end;

procedure TfrmMain.RegisterComputer(Info: TComputerInfo);
begin
  Info.Item := lsvComputers.Items.Add;
  Info.OnGetItem := OnGetAdapterItem;
  Info.OnMessage := OnLogMessageString;
end;

procedure TfrmMain.RefreshComputers;
var
  I: Integer;
begin
  with lsvComputers do
    for I := 0 to Items.Count - 1 do
      if Assigned(Items[I].Data) then
        TComputerInfo(Items[I].Data).Refresh(True);
end;

procedure TfrmMain.ComputerAddHostActionExecute(Sender: TObject);
var
  Info: TComputerInfo;
  APcName: string;
  ASvcPwd: string;
  APort: Word;
begin
  Info := TComputerInfo.Create('', '', RipDefPort);
  APort := RipDefPort;
  if PcConfig(APcName, ASvcPwd, APort) then
  begin
    Info.Port := APort;
    Info.PcName := APcName;
    Info.SvcPwd := ASvcPwd;
    RegisterComputer(Info);
    Changed := True;
  end else
    Info.Free;
end;

procedure TfrmMain.ComputerDeleteHostActionExecute(Sender: TObject);
var
  Info: TComputerInfo;
begin
  if Assigned(lsvComputers.Selected) and
     Assigned(lsvComputers.Selected.Data) then
  begin
    Info := lsvComputers.Selected.Data;
    if MessageBox(Handle, PChar(Format(SCommonConfirm, [SDeleteComputer, Info.PcName])),
      PChar(Caption), MB_ICONQUESTION + MB_DEFBUTTON2  + MB_YESNO) = IDYES then
    begin
      Info.Free;
      lsvComputers.Selected.Delete;
      Changed := True;
    end;
  end;
end;

procedure TfrmMain.ComputerPropertiesActionExecute(Sender: TObject);
var
  Info: TComputerInfo;
  APcName: string;
  ASvcPwd: string;
  APort: Word;
begin
  if Assigned(lsvComputers.Selected) and
     Assigned(lsvComputers.Selected.Data) then
  begin
    Info := lsvComputers.Selected.Data;
    APcName := Info.PcName;
    ASvcPwd := Info.SvcPwd;
    APort := Info.Port;
    if PcConfig(APcName, ASvcPwd, APort) then
    begin
      Info.PcName := APcName;
      Info.SvcPwd := ASvcPwd;
      Info.Port := APort;
      Info.UpdateListItem;
      Changed := True;
    end;
  end;
end;

procedure TfrmMain.ComputerDeleteHostActionUpdate(Sender: TObject);
begin
  ComputerDeleteHostAction.Enabled := Assigned(lsvComputers.Selected);
end;

procedure TfrmMain.ComputerPropertiesActionUpdate(Sender: TObject);
begin
  ComputerPropertiesAction.Enabled := Assigned(lsvComputers.Selected);
end;

procedure TfrmMain.lsvComputersDblClick(Sender: TObject);
begin
  if ComputerPropertiesAction.Enabled then
    ComputerPropertiesAction.Execute;
end;

procedure TfrmMain.RefreshActionExecute(Sender: TObject);
begin
  RefreshComputers;
end;

procedure TfrmMain.OnGetAdapterItem(Sender: TObject; var Item: TListItem);
begin
  Item := lsvAdapters.Items.Add;
end;

procedure TfrmMain.lsvAdaptersDblClick(Sender: TObject);
begin
  if AdapterPropertiesAction.Enabled then
    AdapterPropertiesAction.Execute;
end;

procedure TfrmMain.AdapterPropertiesActionUpdate(Sender: TObject);
begin
  AdapterPropertiesAction.Enabled := lsvAdapters.Selected <> nil;
end;

procedure TfrmMain.AdapterPropertiesActionExecute(Sender: TObject);
var
  AdapterInfo: TAdapterInfo;
  RipInfo: TRipAdapterInfo;
begin
  if Assigned(lsvAdapters.Selected) and
     Assigned(lsvAdapters.Selected.Data) then
  begin
    AdapterInfo := TAdapterInfo(lsvAdapters.Selected.Data);
    if not AdapterInfo.Deleted then
    begin
      RipInfo := AdapterInfo.Info;
      if IpConfigure(@RipInfo) then
      begin
        if not AdapterInfo.ConfigureAdapter(@RipInfo) then
        begin
          MessageBox(Handle, PChar(SCannotConfigureIp),
            PChar(Caption), MB_ICONERROR + MB_OK);
        end;
      end;
    end else
      MessageBox(Handle, PChar(SCannotConfigureIp),
        PChar(Caption), MB_ICONERROR + MB_OK);
  end;
end;

procedure TfrmMain.ComputerPingActionExecute(Sender: TObject);
var
  Info: TComputerInfo;
  Ping: array[0..MAX_PATH]of Char;
begin
  if Assigned(lsvComputers.Selected) and
     Assigned(lsvComputers.Selected.Data) then
  begin
    Info := lsvComputers.Selected.Data;
    StrCopy(Ping, 'ping -t ');
    StrCat(Ping, PChar(Info.PcName));
    WinExec(Ping, SW_SHOW);
  end;
end;

procedure TfrmMain.ComputerPingActionUpdate(Sender: TObject);
begin
  ComputerPingAction.Enabled := lsvComputers.Selected <> nil;
end;

procedure TfrmMain.AdapterDisableActionExecute(Sender: TObject);
var
  Info: TAdapterInfo;
begin
  if Assigned(lsvAdapters.Selected) and
     Assigned(lsvAdapters.Selected.Data) then
  begin
    Info := TAdapterInfo(lsvAdapters.Selected.Data);
    if MessageBox(Handle, PChar(Format(SCommonConfirm, [SDisableAdapter, Info.Description])),
      PChar(Caption), MB_ICONQUESTION + MB_YESNO + MB_DEFBUTTON2) = IDYES then
    begin
      if not Info.DisableAdapter then
      begin
        MessageBox(Handle, PChar(SCannotConfigureIp),
          PChar(Caption), MB_ICONERROR + MB_OK);
      end;
    end;
  end;
end;

procedure TfrmMain.AdapterDisableActionUpdate(Sender: TObject);
begin
  AdapterDisableAction.Enabled := lsvAdapters.Selected <> nil;
end;

procedure TfrmMain.AdapterEnableActionExecute(Sender: TObject);
var
  Info: TAdapterInfo;
begin
  if Assigned(lsvAdapters.Selected) and
     Assigned(lsvAdapters.Selected.Data) then
  begin
    Info := TAdapterInfo(lsvAdapters.Selected.Data);
    if MessageBox(Handle, PChar(Format(SCommonConfirm, [SEnableAdapter, Info.Description])),
      PChar(Caption), MB_ICONQUESTION + MB_YESNO + MB_DEFBUTTON2) = IDYES then
    begin
      if not Info.EnableAdapter then
      begin
        MessageBox(Handle, PChar(SCannotConfigureIp),
          PChar(Caption), MB_ICONERROR + MB_OK);
      end;
    end;
  end;
end;

procedure TfrmMain.AdapterEnableActionUpdate(Sender: TObject);
begin
  AdapterEnableAction.Enabled := lsvAdapters.Selected <> nil;
end;

procedure TfrmMain.ComputerShutdownActionExecute(Sender: TObject);
var
  Info: TComputerInfo;
begin
  if Assigned(lsvComputers.Selected) and
     Assigned(lsvComputers.Selected.Data) then
  begin
    Info := lsvComputers.Selected.Data;
    if MessageBox(Handle, PChar(Format(SCommonConfirm, [SShutdownComputer, Info.PcName])),
      PChar(Caption), MB_ICONQUESTION + MB_YESNO + MB_DEFBUTTON2) = IDYES then
    begin
      if not Info.ShutdownWindows(True) then
      begin
        MessageBox(Handle, PChar(SCannotConfigureIp),
          PChar(Caption), MB_ICONERROR + MB_OK);
      end;
    end;
  end;
end;

procedure TfrmMain.ComputerShutdownActionUpdate(Sender: TObject);
begin
  ComputerShutdownAction.Enabled := lsvComputers.Selected <> nil;
end;

procedure TfrmMain.ComputerRestartActionExecute(Sender: TObject);
var
  Info: TComputerInfo;
begin
  if Assigned(lsvComputers.Selected) and
     Assigned(lsvComputers.Selected.Data) then
  begin
    Info := lsvComputers.Selected.Data;
    if MessageBox(Handle, PChar(Format(SCommonConfirm, [SRestartComputer, Info.PcName])),
      PChar(Caption), MB_ICONQUESTION + MB_YESNO + MB_DEFBUTTON2) = IDYES then
    begin
      if not Info.RestartWindows(True) then
      begin
        MessageBox(Handle, PChar(SCannotConfigureIp),
          PChar(Caption), MB_ICONERROR + MB_OK);
      end;
    end;
  end;
end;

procedure TfrmMain.ComputerRestartActionUpdate(Sender: TObject);
begin
  ComputerRestartAction.Enabled := lsvComputers.Selected <> nil;
end;

procedure TfrmMain.OnLogMessageString(Sender: TObject;
  Kind: TMessageKind; Msg: string);
var
  TimeStamp: string;
begin
  if Sender is TComputerInfo then
    with lsvHistory.Items.Add, Sender as TComputerInfo do
    begin
      ImageIndex := 3 + Integer(Kind);
      Caption := KindToStr(Kind);
      DateTimeToString(TimeStamp, 'dd/mm/yyyy hh:nn:ss.zzz', Now());
      SubItems.Add(TimeStamp);
      SubItems.Add(PcName);
      SubItems.Add(Msg);
      MakeVisible(False);
    end;
end;

end.
