{*********************************************************}
{                                                         }
{    RIPC                                                 }
{    Control Console Common Classes                       }
{                                                         }
{    Copyright (c) 2005 Volodimir V. Flyonts              }
{    Send all comments to e-mail: flyonts@ukr.net         }
{                                                         }
{    Version 1.1 Beta 2                                   }
{                                                         }
{*********************************************************}

{                                                     

Copyright (C) 2005 Vladimir V. Flyonts

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston,
MA  02111-1307, USA.

Please send remarks, questions and bug reports to flyonts@ukr.net

}

unit RipUtils;

interface

uses
  Windows, SysUtils, Classes, ComCtrls, ScktComp, WinSock, Md5, IpTypes, RipSvc;

type

  TCardinalArray = packed array [0..1] of Cardinal;

  TComputerInfo = class;

  ERipSvcError = class(Exception);

  TMemoryBuffer = packed array [0..81920] of Char;
  TMessageKind = (mtInfo, mtError, mtSend, mtRecv);

  TOnReceiveEvent = procedure (Sender: TObject; const Recv; Size: Cardinal) of object;
  TOnMessageEvent = procedure (Sender: TObject; Kind: TMessageKind; Status: string) of object;

  TServiceThread = class(TThread)
  private
    FComputer: TComputerInfo;
    FSocket: TClientWinSocket;
    FGetVersion: Boolean;
    FLoginFirst: Boolean;
    FSendBuffer: TMemoryBuffer;
    FSendBufLen: Integer;
    FRecvBuffer: TMemoryBuffer;
    FRecvBufLen: Integer;
    FLastMessage: string;
    FMessageKind: TMessageKind;
    FOnReceive: TOnReceiveEvent;
    FOnMessage: TOnMessageEvent;
    procedure SyncReceive;
    procedure SyncMessage;
    procedure LogMessage(Kind: TMessageKind; Msg: string);
  public
    constructor Create(Sender: TComputerInfo;
      GetVersion, Login: Boolean; const Send; Size: Integer);
    destructor Destroy; override;
    procedure Execute; override;
    property OnReceive: TOnReceiveEvent read FOnReceive write FOnReceive;
    property OnMessage: TOnMessageEvent read FOnMessage write FOnMessage;
  end;

  TAdapterInfo = class
  private
    FInfo: TRipAdapterInfo;
    FItem: TListItem;
    FOwner: TComputerInfo;
    FDeleted: Boolean;
    function GetAdapterName: string;
    function GetDescription: string;
    procedure SetAdapterName(const Value: string);
    procedure SetDescription(const Value: string);
    procedure SetItem(AItem: TListItem);
    procedure SetDeleted(Value: Boolean);
  public
    constructor Create(AOwner: TComputerInfo; AInfo: PRipAdapterInfo);
    destructor Destroy; override;
    function CreateListItem(AListView: TListView): TListItem;
    function ConfigureAdapter(AInfo: PRipAdapterInfo): Boolean;
    function DisableAdapter: Boolean;
    function EnableAdapter: Boolean;
    procedure UpdateListItem;
    procedure SetAdapterInfo(AInfo: PRipAdapterInfo);
    property AdapterName: string read GetAdapterName write SetAdapterName;
    property Description: string read GetDescription write SetDescription;
    property Info: TRipAdapterInfo read FInfo write FInfo;
    property Item: TListItem read FItem write SetItem;
    property Owner: TComputerInfo read FOwner;
    property Deleted: Boolean read FDeleted write SetDeleted;
  end;

  TOnGetListItemEvent = procedure (Sender: TObject; var Item: TListItem) of object;

  TComputerInfo = class
  private
    FPcName: string;
    FSvcPwd: string;
    FPort: Word;
    FItem: TListItem;
    FAdapters: TList;
    FSvcVer: string;
    FStatus: string;
    FFixedInfo: TRipFixedInfo;
    FSocketThread: TServiceThread;
    FOnGetItem: TOnGetListItemEvent;
    FOnMessage: TOnMessageEvent;
    function GetAdapterInfo(Index: Integer): TAdapterInfo;
    function GetAdaptersCount: Integer;
    function ExecuteCommand(const Command; Size: Cardinal;
      Async: Boolean = False): Boolean; 
    procedure SetItem(AItem: TListItem);
    procedure OnReceiveData(Sender: TObject; const Recv; Size: Cardinal);
    procedure OnLogMessage(Sender: TObject; Kind: TMessageKind; Msg: string);
    procedure OnTerminateThread(Sender: TObject);
  public
    { Public declarations }
    constructor Create(const APcName, ASvcPwd: string; APort: Cardinal);
    destructor Destroy; override;
    function ShutdownWindows(Async: Boolean = False): Boolean;
    function RestartWindows(Async: Boolean = False): Boolean;
    function ConfigureAdapter(Sender: TAdapterInfo;
      Info: PRipAdapterInfo; Async: Boolean = False): Boolean;
    function DisableAdapter(Sender: TAdapterInfo; AdapterName: PChar;
      Async: Boolean = False): Boolean;
    function EnableAdapter(Sender: TAdapterInfo; AdapterName: PChar;
      Async: Boolean = False): Boolean;
    function Refresh(Async: Boolean = False): Boolean;
    function Read(const Source): Cardinal;
    function Write(var Dest): Cardinal;
    function FindAdapter(const AdapterName: string): Integer;
    procedure DeleteAdapter(Index: Integer);
    procedure SetNetworkInfo(Info: PRipNetworkInfo);
    procedure UpdateListItem;
    procedure UpdateAdapterInfo(Info: PRipAdapterInfo);
    property Adapters[Index: Integer]: TAdapterInfo read GetAdapterInfo;
    property AdaptersCount: Integer read GetAdaptersCount;
    property PcName: string read FPcName write FPcName;
    property SvcPwd: string read FSvcPwd write FSvcPwd;
    property Port: Word read FPort write FPort;
    property SvcVer: string read FSvcVer write FSvcVer;
    property Status: string read FStatus write FStatus;
    property FixedInfo: TRipFixedInfo read FFixedInfo;
    property Item: TListItem read FItem write SetItem;
    property OnGetItem: TOnGetListItemEvent
      read FOnGetItem write FOnGetItem;
    property OnMessage: TOnMessageEvent
      read FOnMessage write FOnMessage;
  end;

function KindToStr(Kind: TMessageKind): string;
function AnsverCode(Buffer: PChar): Cardinal;
  
const

  PcMark = $4350;
  DefTimeOut =  5000;
  MaxTimeOut = 30000;

implementation

resourcestring

  SRefreshing = 'Refreshing...';
  SReceiveError = 'Receive Error: %s';
  SSendError = 'Send Error: %s';
  SServiceError = 'Service Error: %s';
  SConnectTo = 'Connect to %s:%d';
  SConnectError = 'Connect Error: %s';
  SConnected = 'Connected, wait for welcome message...';

  SSuccess = 'Command completed successfully.';
  SNotSigned = 'Data not signed.';
  SReceiveBinary = 'Receive binary data.';

function KindToStr(Kind: TMessageKind): string;
begin
  case Kind of
  mtInfo:  Result := 'Info';
  mtError: Result := 'Error';
  mtSend:  Result := 'Send';
  mtRecv:  Result := 'Recv';
  end;
end;

function AnsverCode(Buffer: PChar): Cardinal;
begin
  Result := PCardinal(Buffer)^ and $00ffffff;
end;

{ TAdapterInfo }

function TAdapterInfo.ConfigureAdapter(AInfo: PRipAdapterInfo): Boolean;
begin
  if FOwner = nil then
  begin
    Result := False;
    Exit;
  end;

  Result := FOwner.ConfigureAdapter(Self, AInfo, True);

  if Result then
    SetAdapterInfo(AInfo);

end;

constructor TAdapterInfo.Create(AOwner: TComputerInfo; AInfo: PRipAdapterInfo);
begin
  inherited Create;
  FDeleted := False;
  FOwner := AOwner;
  if AInfo <> nil then
    Move(AInfo^, FInfo, SizeOf(TRipAdapterInfo));
end;

function TAdapterInfo.CreateListItem(AListView: TListView): TListItem;
begin
  if not Assigned(AListView) then
  begin
    Result := FItem;
    Exit;
  end;

  if Assigned(FItem) then
    FreeAndNil(FItem);
    
  FItem := AListView.Items.Add;
  FItem.Data := Self;
  UpdateListItem;
  Result := FItem;
end;

destructor TAdapterInfo.Destroy;
begin
  if FItem <> nil then
    FItem.Delete;
  inherited;
end;

function TAdapterInfo.DisableAdapter: Boolean;
begin
  if FOwner = nil then
  begin
    Result := False;
    Exit;
  end;

  Result := FOwner.DisableAdapter(Self, FInfo.AdapterName, True);
end;

function TAdapterInfo.EnableAdapter: Boolean;
begin
  if FOwner = nil then
  begin
    Result := False;
    Exit;
  end;

  Result := FOwner.EnableAdapter(Self, FInfo.AdapterName, True);
end;

function TAdapterInfo.GetAdapterName: string;
begin
  SetString(Result, PChar(@FInfo.AdapterName), StrLen(FInfo.AdapterName));
end;

function TAdapterInfo.GetDescription: string;
begin
  SetString(Result, PChar(@FInfo.Description), StrLen(FInfo.Description));
end;

procedure TAdapterInfo.SetAdapterInfo(AInfo: PRipAdapterInfo);
begin
  if AInfo = nil then
    Exit;
  Move(AInfo^, FInfo, SizeOf(TRipAdapterInfo));
  UpdateListItem;
end;

procedure TAdapterInfo.SetAdapterName(const Value: string);
begin
  FillChar(FInfo.AdapterName, SizeOf(FInfo.AdapterName), 0);
  StrLCopy(FInfo.AdapterName, PChar(Value), MAX_ADAPTER_NAME_LENGTH);
end;

procedure TAdapterInfo.SetDeleted(Value: Boolean);
begin
  if FDeleted <> Value then
  begin
    FDeleted := Value;
    UpdateListItem;
  end;
end;

procedure TAdapterInfo.SetDescription(const Value: string);
begin
  FillChar(FInfo.Description, SizeOf(FInfo.Description), 0);
  StrLCopy(FInfo.Description, PChar(Value), MAX_ADAPTER_DESCRIPTION_LENGTH);
end;

procedure TAdapterInfo.SetItem(AItem: TListItem);
begin
  if FItem <> nil then
    FItem.Data := nil;

  FItem := AItem;
  
  if FItem <> nil then
    FItem.Data := Self;
    
  UpdateListItem;
end;

procedure TAdapterInfo.UpdateListItem;
begin
  if FItem = nil then
    Exit;

  with FItem, FInfo do
  begin
    if FDeleted then
      ImageIndex := 2
    else
      ImageIndex := 1;
      
    Caption := Description;
    while SubItems.Count < 8 do
      SubItems.Add('');

    SubItems[0] := AdapterName;
    if FOwner <> nil then
    SubItems[1] := FOwner.PcName;
    SubItems[2] := PhysAddrToStr(Address, AddressLength);
    SubItems[3] := YesNo(DhcpEnabled);
    SubItems[4] := IpListToStr(@IpAddressList);
    SubItems[5] := IpListToStr(@IpAddressList, True);
    SubItems[6] := IpListToStr(@GatewayList);
    SubItems[7] := IpListToStr(@DnsServerList);

    Update;
  end;
end;

{ TComputerInfo }

constructor TComputerInfo.Create(const APcName, ASvcPwd: string; APort: Cardinal);
begin
  inherited Create;
  FPcName := APcName;
  FSvcPwd := ASvcPwd;
  FPort := APort;
  FAdapters := TList.Create;
end;

destructor TComputerInfo.Destroy;
begin
  if Assigned(FSocketThread) then
    FSocketThread.Free;

  while AdaptersCount > 0 do
    DeleteAdapter(0);
  FAdapters.Destroy;
  
  inherited;
end;

function TComputerInfo.FindAdapter(const AdapterName: string): Integer;
begin
  for Result := 0 to AdaptersCount - 1 do
    if CompareStr(AdapterName, Adapters[Result].AdapterName) = 0 then
      Exit;
  Result := -1;
end;

//
//  Port: Word 
//  PcNameLen: Byte
//  SvcPwdLen: Byte
//  PcName: array[PcNameLen]of Char
//  SvcPwd: array[SvcPwdLen]of Char
//

function TComputerInfo.Read(const Source): Cardinal;
const
  Offset = 0;
var
  PcL, SpL: Byte;
begin
  Move(TByteArray(Source)[Offset], FPort, SizeOf(Word));
  PcL := TByteArray(Source)[Offset+2];
  SpL := TByteArray(Source)[Offset+3];
  SetString(FPcName, PChar(@TByteArray(Source)[Offset+4]), PcL);
  SetString(FSvcPwd, PChar(@TByteArray(Source)[Offset+4+PcL]), SpL);
  Result := 4 + PcL + SpL;
end;

function TComputerInfo.Refresh(Async: Boolean): Boolean;
var
  Index: Integer;
  Command: Cardinal;
begin
  OnLogMessage(Self, mtInfo, SRefreshing);

  // Invalidate
  for Index := 0 to AdaptersCount - 1 do
    Adapters[Index].Deleted := True;

  // Refresh...
  if Assigned(FSocketThread) then
    FSocketThread.Free;

  Command := RIPC_LSB;
  FSocketThread := TServiceThread.Create(Self, True, True, Command, SizeOf(Command));
  FSocketThread.OnMessage := OnLogMessage;
  FSocketThread.OnReceive := OnReceiveData;
  FSocketThread.OnTerminate := OnTerminateThread;
  FSocketThread.Suspended := False;

  if not Async then
  begin
    WaitForSingleObject(FSocketThread.Handle, INFINITE);
    FreeAndNil(FSocketThread);
  end;

  Result := True;
end;

function TComputerInfo.Write(var Dest): Cardinal;
const
  Offset = 0;
var
  PcL, SpL: Byte;
begin
  Move(FPort, TByteArray(Dest)[Offset], SizeOf(Word));
  PcL := Length(FPcName);
  SpL := Length(FSvcPwd);
  TByteArray(Dest)[Offset+2] := Byte(PcL);
  TByteArray(Dest)[Offset+3] := Byte(SpL);
  Move(Pointer(FPcName)^, TByteArray(Dest)[Offset+4], PcL);
  Move(Pointer(FSvcPwd)^, TByteArray(Dest)[Offset+4+PcL], SpL);
  Result := 4 + PcL + SpL;
end;

procedure TComputerInfo.UpdateListItem;
begin
  if Assigned(FItem) then
    with FItem, FFixedInfo do
    begin
      ImageIndex := 0;
      Caption := PcName;

      while SubItems.Count < 9 do
        SubItems.Add('');

      SubItems[0] := FStatus;
      SubItems[1] := string(HostName);
      SubItems[2] := string(DomainName);
      SubItems[3] := IpListToStr(@DnsServerList);
      SubItems[4] := NodeTypeToStr(NodeType);
      SubItems[5] := string(ScopeId);
      SubItems[6] := YesNo(EnableRouting);
      SubItems[7] := YesNo(EnableProxy);
      SubItems[8] := YesNo(EnableDns);

      Update;
    end;
end;

procedure TComputerInfo.UpdateAdapterInfo(Info: PRipAdapterInfo);
var
  ListItem: TListItem;
  Index: Integer;
  Adapter: TAdapterInfo;
begin
  if Info = nil then
    Exit;
  Index := FindAdapter(string(Info^.AdapterName));
  if Index >= 0 then
  begin
    Adapter := Adapters[Index];
    Adapter.SetAdapterInfo(Info);
    Adapter.Deleted := False;
  end
  else
  begin
    Adapter := TAdapterInfo.Create(Self, Info);
    FAdapters.Add(Adapter);
    ListItem := nil;
    if Assigned(FOnGetItem) then
      FOnGetItem(Self, ListItem);
    Adapter.Item := ListItem;
  end;
end;

procedure TComputerInfo.SetItem(AItem: TListItem);
begin
  if FItem <> nil then
    FItem.Data := nil;

  FItem := AItem;
  FItem.Data := Self;
  UpdateListItem;
end;

procedure TComputerInfo.SetNetworkInfo(Info: PRipNetworkInfo);
var
  Index, Size: Integer;
  Sign: TMD5Digest;
begin
  if Info = nil then
    Exit;
  if IsBadReadPtr(Info, SizeOf(TRipNetworkInfo)) then
    Exit;

  with Info^.Header do
    Size := AdapterSize * AdapterCount + FixedSize;
  Sign := MD5Buffer(Info^.FixedInfo, Size);
  if MD5DigestCompare(Sign, Info^.Header.Sign) then
  begin
    Move(Info^.FixedInfo, FFixedInfo, SizeOf(FFixedInfo));

    for Index := 0 to Info^.Header.AdapterCount - 1 do
      UpdateAdapterInfo(@Info^.Adapters[Index]);

    OnLogMessage(Self, mtInfo, SSuccess);
  end else
    OnLogMessage(Self, mtError, SNotSigned);
end;

function TComputerInfo.GetAdapterInfo(Index: Integer): TAdapterInfo;
begin
  Result := TAdapterInfo(FAdapters.Items[Index]);
end;

function TComputerInfo.GetAdaptersCount: Integer;
begin
  Result := FAdapters.Count;
end;

procedure TComputerInfo.DeleteAdapter(Index: Integer);
begin
  Adapters[Index].Destroy;
  FAdapters.Delete(Index);
end;

function TComputerInfo.ConfigureAdapter(Sender: TAdapterInfo;
  Info: PRipAdapterInfo; Async: Boolean): Boolean;
var
  Command: TRipIpConfigure;
begin
  if Info = nil then
  begin
    Result := False;
    Exit;
  end;

  Move(Info^, Command.Adapter, SizeOf(TRipAdapterInfo));

  with Command.Header do
  begin
    Status := RIPC_IPC;
    HeaderSize := SizeOf(TRipInfoHeader);
    FixedSize := 0;
    AdapterSize := SizeOf(TRipAdapterInfo);
    AdapterCount := 1;
    TotalSize := HeaderSize + FixedSize + AdapterCount * AdapterSize;
    Sign := MD5Buffer(Command.Adapter, AdapterCount * AdapterSize);
  end;

  Result := ExecuteCommand(Command, SizeOf(Command), Async);
end;

function TComputerInfo.DisableAdapter(Sender: TAdapterInfo;
  AdapterName: PChar; Async: Boolean): Boolean;
var
  Command: array[0..260]of Char;
begin
  FillChar(Command, SizeOf(Command), 0);
  PCardinal(@Command)^ := RIPC_DIS;
  StrCopy(@Command[4], AdapterName);

  Result := ExecuteCommand(Command, SizeOf(Command), Async);
end;

function TComputerInfo.EnableAdapter(Sender: TAdapterInfo;
  AdapterName: PChar; Async: Boolean): Boolean;
var
  Command: array[0..260]of Char;
begin
  FillChar(Command, SizeOf(Command), 0);
  PCardinal(@Command)^ := RIPC_ENA;
  StrCopy(@Command[4], AdapterName);

  Result := ExecuteCommand(Command, SizeOf(Command), Async);
end;

function TComputerInfo.RestartWindows(Async: Boolean): Boolean;
var
  Command: Cardinal;
begin
  Command := RIPC_RSW;
  Result := ExecuteCommand(Command, SizeOf(Command), Async);
end;

function TComputerInfo.ShutdownWindows(Async: Boolean): Boolean;
var
  Command: Cardinal;
begin
  Command := RIPC_RSW;
  Result := ExecuteCommand(Command, SizeOf(Command), Async);
end;

function TComputerInfo.ExecuteCommand(const Command; Size: Cardinal;
  Async: Boolean): Boolean;
begin
  if Assigned(FSocketThread) then
    FSocketThread.Free;

  FSocketThread := TServiceThread.Create(Self, False, True, Command, Size);
  FSocketThread.OnMessage := OnLogMessage;
  FSocketThread.OnReceive := OnReceiveData;
  FSocketThread.OnTerminate := OnTerminateThread;
  FSocketThread.Suspended := False;

  if not Async then
  begin
    WaitForSingleObject(FSocketThread.Handle, INFINITE);
    FreeAndNil(FSocketThread);
  end;
  
  Result := True;
end;

procedure TComputerInfo.OnLogMessage(Sender: TObject;
  Kind: TMessageKind; Msg: string);
begin
  FStatus := Msg;
  UpdateListItem;
  if Assigned(FOnMessage) then
    FOnMessage(Self, Kind, Msg);
end;

procedure TComputerInfo.OnReceiveData(Sender: TObject;
  const Recv; Size: Cardinal);
begin
  if (Size = 0) or (Size = Cardinal(SOCKET_ERROR)) then
    Exit;
    
  if Cardinal(Recv) = RIPC_VER1 then
  begin
    SetString(FSvcVer, PChar(@Recv), Size);
    UpdateListItem;
  end
    else
  if Cardinal(Recv) = 0 then { data }
  begin
    SetNetworkInfo(@Recv);
    UpdateListItem;
  end;
end;

procedure TComputerInfo.OnTerminateThread(Sender: TObject);
begin
//  if Sender = FSocketThread then
//    FreeAndNil(FSocketThread);
end;

{ TServiceThread }

constructor TServiceThread.Create(Sender: TComputerInfo;
  GetVersion, Login: Boolean; const Send; Size: Integer);
begin
  inherited Create(True);
  FComputer := Sender;
  FGetVersion := GetVersion;
  FLoginFirst := Login;
  Move(Send, FSendBuffer, Size);
  FSendBufLen := Size;
end;

destructor TServiceThread.Destroy;
begin
  if not Terminated then
    Terminate;
  if Assigned(FSocket) then
    FSocket.Free;
  inherited;
end;

procedure TServiceThread.Execute;
var
  Buffer: array[0..256]of Char;
  BufLen: Integer;
  Str: string;
  Port: Word;
begin
  if Assigned(FSocket) then
    FSocket.Free;

  FSocket := TClientWinSocket.Create(INVALID_SOCKET);
  with FSocket do
  try
    Str := FComputer.PcName;
    Port := FComputer.Port;
    LogMessage(mtInfo, Format(SConnectTo, [Str, Port]));
    ClientType := ctBlocking;
    Open(Str, '', '', Port, True);
    if not Connected then
      raise ESocketError.CreateFmt(SConnectError, [SysErrorMessage(WSAGetLastError)]);
    LogMessage(mtInfo, SConnected);

    // Setup timeout
    BufLen := DefTimeOut;
    setsockopt(SocketHandle, SOL_SOCKET, SO_SNDTIMEO, PChar(@BufLen), SizeOf(BufLen)); 
    setsockopt(SocketHandle, SOL_SOCKET, SO_RCVTIMEO, PChar(@BufLen), SizeOf(BufLen)); 

    // Wait for '220-Service ready for new user.'
    FRecvBufLen := ReceiveBuf(FRecvBuffer, SizeOf(FRecvBuffer));
    if FRecvBufLen <= 0 then
      raise ESocketError.CreateFmt(SReceiveError, [SysErrorMessage(WSAGetLastError)]);
    SetString(Str, PChar(@FRecvBuffer), FRecvBufLen);
    if AnsverCode(FRecvBuffer) <> RIPC_220 then
      raise ERipSvcError.CreateFmt(SServiceError, [Str]);
    LogMessage(mtRecv, Str);
    Synchronize(SyncReceive);

    if FGetVersion then
    begin
      LogMessage(mtSend, 'VER');
      StrCopy(Buffer, 'VER');
      // Send 'VER'
      BufLen := SendBuf(Buffer, StrLen(Buffer));
      if BufLen <= 0 then
        raise ESocketError.CreateFmt(SSendError, [SysErrorMessage(WSAGetLastError)]);
      // Wait for 'RIPC/1.1 ...'
      FRecvBufLen := ReceiveBuf(FRecvBuffer, SizeOf(FRecvBuffer));
      if FRecvBufLen <= 0 then
        raise ESocketError.CreateFmt(SReceiveError, [SysErrorMessage(WSAGetLastError)]);
      SetString(Str, PChar(@FRecvBuffer), FRecvBufLen);
      LogMessage(mtRecv, Str);
      Synchronize(SyncReceive);
    end;

    if FLoginFirst then
    begin
      LogMessage(mtSend, 'PAS');
      StrCopy(Buffer, 'PAS ');
      Str := FComputer.SvcPwd;
      StrCat(Buffer, PChar(Str));
      // Send 'PAS <password>'
      BufLen := SendBuf(Buffer, StrLen(Buffer));
      if BufLen <= 0 then
        raise ESocketError.CreateFmt(SSendError, [SysErrorMessage(WSAGetLastError)]);
      // Wait for '230-User logged in, proceed.'
      FRecvBufLen := ReceiveBuf(FRecvBuffer, SizeOf(FRecvBuffer));
      if FRecvBufLen <= 0 then
        raise ESocketError.CreateFmt(SReceiveError, [SysErrorMessage(WSAGetLastError)]);
      SetString(Str, PChar(@FRecvBuffer), FRecvBufLen);
      if AnsverCode(FRecvBuffer) <> RIPC_230 then
        raise ERipSvcError.CreateFmt(SServiceError, [Str]);
      LogMessage(mtRecv, Str);
      Synchronize(SyncReceive);
    end;

    // Send Control Command
    SetString(Str, PChar(@FSendBuffer), 4);
    LogMessage(mtSend, Str);
    BufLen := SendBuf(FSendBuffer, FSendBufLen);
    if BufLen < FSendBufLen then
      raise ESocketError.CreateFmt(SSendError, [SysErrorMessage(WSAGetLastError)]);
    // Recevie Ansver
    FRecvBufLen := ReceiveBuf(FRecvBuffer, SizeOf(FRecvBuffer));
    if FRecvBufLen <= 0 then
      raise ESocketError.CreateFmt(SReceiveError, [SysErrorMessage(WSAGetLastError)]);

    if FRecvBuffer[0] <> #0 then
    begin
      SetString(Str, PChar(@FRecvBuffer), FRecvBufLen);
      LogMessage(mtRecv, Str);
    end else
      LogMessage(mtRecv, SReceiveBinary);
    
    Synchronize(SyncReceive);
    
  except
    on E: Exception do LogMessage(mtError, E.Message);
  end;

  FreeAndNil(FSocket);
end;

procedure TServiceThread.LogMessage(Kind: TMessageKind; Msg: string);
begin
  FMessageKind := Kind;
  FLastMessage := Msg;
  Synchronize(SyncMessage);
end;

procedure TServiceThread.SyncMessage;
begin
  if Assigned(FOnMessage) then
    FOnMessage(Self, FMessageKind, FLastMessage);
end;

procedure TServiceThread.SyncReceive;
begin
  if Assigned(FOnReceive) then
    FOnReceive(Self, FRecvBuffer, FRecvBufLen);
end;

end.
