{*********************************************************}
{                                                         }
{    RIPC                                                 }
{    Remote IP Config Windows NT Service                  }
{                                                         }
{    Copyright (c) 2005 Volodimir V. Flyonts              }
{    Send all comments to e-mail: flyonts@ukr.net         }
{                                                         }
{    Version 1.1 Beta                                     }
{                                                         }
{*********************************************************}

{

Copyright (C) 2005 Vladimir V. Flyonts

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston,
MA  02111-1307, USA.

Please send remarks, questions and bug reports to flyonts@ukr.net

}

program ripntsvc;

uses
  Windows, WinSvc, WinSock, SysUtils, MD5,
  IpTypes, IpExport, IpHlpApi, IpIfConst, IpRtrMib, SetupApi,
  RipSvc;

{DEFINE _DEBUG}
{DEFINE _NET_DEBUG}
{$DEFINE _LOG_ERROR}
{DEFINE _LOG_DEBUG}

{DEFINE _USE_RTL_CRITICAL_SECTION}

{$R *.res}

const

  EOL = #13#10;

  RipServiceVer = 'RIPC/1.1';

  RipServiceTitle = 'Remote IP Config';
  RipServiceName = 'RIPC';

  { Adapter }

  TcpIpInt = 'SYSTEM\CurrentControlSet\Services\Tcpip\Parameters\Interfaces\';

  { Registry }

  RipRegKeyRIP = 'SOFTWARE\RemoteIP';
  RipRegKeyRipSvc = RipRegKeyRIP + '\Service';
  RipRegValPort = 'Port';
  RipRegValUser = 'User';
  RipRegValAdmin = 'Admin';

  { RipLogin }

  RIP_NONE  = 0;
  RIP_ADMIN = 1;
  RIP_USER  = 2;

  { command line params }

  param_ntservice = '--ntservice';

  param_i         = '-i';
  param_install   = '--install';
  param_r         = '-r';
  param_uninstall = '--uninstall';

  param_s         = '-s';
  param_start     = '--start';
  param_t         = '-t';
  param_stop      = '--stop';

  param_a         = '-a';
  param_admin     = '--admin';
  param_u         = '-u';
  param_user      = '--user';
  param_p         = '-p';
  param_port      = '--port';

var

  { WinSvc }
  RipServiceStatus: _SERVICE_STATUS;
  RipStatusHandle: SERVICE_STATUS_HANDLE;

  { Settings }
  RipPort: Integer;
  RipUserPwd: TMD5Digest;
  RipAdminPwd: TMD5Digest;

  { WinSock }
  RipWSAData: TWSAData;
  RipWSASocket: TSOCKET;
  RipSockAddr: TSockAddr;

  { Threads }
  RipThread: THandle;
  RipThreadId: Cardinal;

resourcestring

  FixedInfoFmt = 'Windows IP Configuration'+EOL+
    EOL+
    '        Host Name . . . . . . . . . . . . : %s'+EOL+
    '        Primary Dns Suffix  . . . . . . . : %s'+EOL+
    '        Node Type . . . . . . . . . . . . : %s'+EOL+
    '        IP Routing Enabled. . . . . . . . : %s'+EOL+
    '        WINS Proxy Enabled. . . . . . . . : %s'+EOL+
    EOL;

  AdapterInfoFmt = '%s adapter %s'+EOL+
    EOL+
    '        Connection-specific DNS Suffix  . : %s'+EOL+
    '        Description . . . . . . . . . . . : %s'+EOL+
    '        Physical Address. . . . . . . . . : %s'+EOL+
    '        Dhcp Enabled. . . . . . . . . . . : %s'+EOL+
    '        IP Address. . . . . . . . . . . . : %s'+EOL+
    '        Subnet Mask . . . . . . . . . . . : %s'+EOL+
    '        Default Gateway . . . . . . . . . : %s'+EOL+
    '        DNS Servers . . . . . . . . . . . : %s'+EOL+
    EOL;

{$IFDEF _USE_RTL_CRITICAL_SECTION}
var
  RipLogSection: TRTLCriticalSection;
{$ENDIF _USE_RTL_CRITICAL_SECTION}

{--------------------------------------------------------------------}

procedure LogString(const Str: string);
var
  TimeStamp: string;
  LogFile: Text;
begin
  {$IFDEF _USE_RTL_CRITICAL_SECTION}
  try
    EnterCriticalSection(RipLogSection);
  except
    { In low memory situations, EnterCriticalSection can raise an exception. }
  end;
  {$ENDIF _USE_RTL_CRITICAL_SECTION}

  AssignFile(LogFile, ChangeFileExt(ParamStr(0), '.log'));

  {$I-}
  Append(LogFile);
  if IOResult <> 0 then
    Rewrite(LogFile);
  if IOResult = 0 then
  begin
    DateTimeToString(TimeStamp, '[dd/mm/yyyy hh:nn:ss.zzz] ', Now());
    WriteLn(LogFile, Concat(TimeStamp, StringReplace(Str, EOL, EOL+TimeStamp, [rfReplaceAll])));
    CloseFile(LogFile);
  end;
  {$I+}

  {$IFDEF _USE_RTL_CRITICAL_SECTION}
  LeaveCriticalSection(RipLogSection);
  {$ENDIF _USE_RTL_CRITICAL_SECTION}
end;

procedure LogMessage(const Fmt: string; Args: array of const);
begin
  LogString(Format(Fmt, Args));
end;

procedure LogError(const Fmt: string; Args: array of const);
begin
{$IFDEF _LOG_ERROR}
  LogString(Concat('Error:',' ',Format(Fmt, Args)));
{$ENDIF _LOG_ERROR}
end;

procedure LogDebug(const Fmt: string; Args: array of const);
begin
{$IFDEF _LOG_DEBUG}
  LogString(Concat('Debug:',' ',Format(Fmt, Args)));
{$ENDIF _LOG_DEBUG}
end;

{--------------------------------------------------------------------}

function MoveIp(Ip: PIpAddrString; List: PIpAddrList): Cardinal;
begin
  Result := 0;
  if List = nil then
    Exit;
  while (Ip <> nil) and (Result < MAX_ADAPTER_COUNT) do
  begin
    Move(Ip^, List^.Adrs[Result], SizeOf(Ip^));
    Ip := Ip^.Next;
    Inc(Result);
  end;
  List^.Count := Result;
end;

function RipIntSystemShutdown(Msg: PChar; Timeout: Cardinal; Reboot: Bool): Bool;
const
  SE_SHUTDOWN_NAME = 'SeShutdownPrivilege';
var
  RetLen: Cardinal;
  Token: THANDLE;
  tkp: TOKEN_PRIVILEGES;
begin
   // Get the current process token handle so we can get shutdown
   // privilege.
  Result := OpenProcessToken(GetCurrentProcess(),
      TOKEN_ADJUST_PRIVILEGES or TOKEN_QUERY, Token);
  if not Result then
    Exit;

  // Get the LUID for shutdown privilege.
  LookupPrivilegeValue(nil, SE_SHUTDOWN_NAME, tkp.Privileges[0].Luid);

  tkp.PrivilegeCount := 1;  // one privilege to set
  tkp.Privileges[0].Attributes := SE_PRIVILEGE_ENABLED;

  // Get shutdown privilege for this process.
  RetLen := 0;
  AdjustTokenPrivileges(Token, FALSE, tkp, 0, nil, RetLen);

  // Cannot test the return value of AdjustTokenPrivileges.
  if GetLastError() <> ERROR_SUCCESS then
  begin
    Result := False;
    Exit;
  end;

  Result := InitiateSystemShutdown(
      nil,     // shut down local computer
      Msg,     // message for user
      Timeout, // time-out period
      FALSE,   // ask user to close apps
      Reboot); // reboot after shutdown

  if not Result then
    Exit;

   // Disable shutdown privilege. 
   tkp.Privileges[0].Attributes := 0;
   RetLen := 0; 
   AdjustTokenPrivileges(Token, FALSE, tkp, 0, nil, RetLen);
end;

function RipIntFindAdapter(
  const AdapterName: PChar;
  var MemberIndex: Integer;
  var InfoSet: HDEVINFO;
  var InfoData: SP_DEVINFO_DATA): Cardinal;
const
  NetCfgInstanceId : PChar = 'NetCfgInstanceId';
  ClassGUID: array[0..15]of Byte =
    ($72,$E9,$36,$4D,$25,$E3,$CE,$11,$BF,$C1,$08,$00,$2B,$E1,$03,$18);
var
  DeviceRegKey: HKEY;
  szInstanceId: array[0..128]of Char;
  dwValueSize: Cardinal;
begin
  if InfoSet = nil then
    InfoSet := SetupDiGetClassDevs(@ClassGUID, nil, 0, DIGCF_PRESENT);

  if THANDLE(InfoSet) = INVALID_HANDLE_VALUE then
  begin
    Result := GetLastError;
    Exit;
  end;

  Result := ERROR_SUCCESS;

  repeat
  
    if not SetupDiEnumDeviceInfo(InfoSet, MemberIndex, InfoData) then
    begin
      Result := GetLastError;
      Break;
    end;

    DeviceRegKey := SetupDiOpenDevRegKey(InfoSet, InfoData,
      DICS_FLAG_GLOBAL, 0, DIREG_DRV, KEY_READ);

    if DeviceRegKey = 0 then
    begin
      Result := GetLastError;
      Break;
    end;

    FillChar(szInstanceId, SizeOf(szInstanceId), 0);
    dwValueSize := SizeOf(szInstanceId)-1;
    Result := RegQueryValueEx(DeviceRegKey, NetCfgInstanceId,
      nil, nil,
      @szInstanceId, @dwValueSize);

    RegCloseKey(DeviceRegKey);  

    if Result <> NO_ERROR then
      Break;  

    Inc(MemberIndex);

  until StrComp(AdapterName, szInstanceId) = 0;

end;

function RipIntCallClassInstaller(
  const InfoSet: HDEVINFO;
  InfoData: PSPDevInfoData;
  State: DWORD): BOOL;
var
  Params: SP_PROPCHANGE_PARAMS;
begin
  FillChar(Params, SizeOf(Params), 0);
  with Params do
  begin
    ClassInstallHeader.cbSize := SizeOf(SP_CLASSINSTALL_HEADER);
    ClassInstallHeader.InstallFunction := DIF_PROPERTYCHANGE;
    StateChange := State;
    Scope := DICS_FLAG_GLOBAL;
  end;

  Result := SetupDiSetClassInstallParams(InfoSet, InfoData, @Params, SizeOf(Params)) and
    SetupDiCallClassInstaller(DIF_PROPERTYCHANGE, InfoSet, InfoData);
end;

function RipIntAdapterControl(const Adapter: PChar; Control: Cardinal): Cardinal;
var
  InfoSet: HDEVINFO;
  InfoData: SP_DEVINFO_DATA;
  MemberIndex: Integer;
begin
  InfoSet := nil;
  FillChar(InfoData, SizeOf(InfoData), 0);
  InfoData.cbSize := SizeOf(SP_DEVINFO_DATA);

  MemberIndex := 0;

  Result := RipIntFindAdapter(Adapter, MemberIndex, InfoSet, InfoData);
  if Result = ERROR_SUCCESS then
  begin
    if not RipIntCallClassInstaller(InfoSet, @InfoData, Control) then
      Result := GetLastError;
  end;

  SetupDiDestroyDeviceInfoList(InfoSet);
end;

function RipIntAdapterEnable(const AdapterName: PChar): Cardinal;
begin
  Result := RipIntAdapterControl(AdapterName, DICS_ENABLE); 
end;

function RipIntAdapterDisable(const AdapterName: PChar): Cardinal;
begin
  Result := RipIntAdapterControl(AdapterName, DICS_DISABLE);
end;

function RipIntAdapterUpdate(Info: PRipAdapterInfo): Cardinal;
const
  EnableDHCP = 'EnableDHCP';
  IPAddress = 'IPAddress';
  SubnetMask = 'SubnetMask';
  DefaultGateway = 'DefaultGateway';
  NameServer = 'NameServer';
  DhcpServer = 'DhcpServer';
  DhcpIpAddress = 'DhcpIpAddress';
  DhcpSubnetMask = 'DhcpSubnetMask';
  DhcpNameServer = 'DhcpNameServer';
  DhcpDefaultGateway = 'DhcpDefaultGateway';
  DhcpRetryTime = 'DhcpRetryTime';
  DhcpRetryStatus = 'DhcpRetryStatus';
  ZeroIp = '0.0.0.0'#0#0#0;
  ZeroIpLen = 10;
  MaxIp = '255.255.255.255'#0#0#0;
  MaxIpLen = 18;

var
  Key: HKEY;
  DhcpEnabled: Boolean;
  Value: array[0..4098]of Char;
  Size: Cardinal;
  
begin
  if Info = nil then
  begin
    Result := ERROR_INVALID_PARAMETER;
    Exit;
  end;
  
  StrCopy(Value, TcpIpInt);
  StrCat(Value, Info^.AdapterName);

  Result := RegOpenKeyEx(HKEY_LOCAL_MACHINE, Value, 0, KEY_WRITE, Key);
  if Result <> ERROR_SUCCESS then
    Exit;

  RegSetValueEx(Key, EnableDHCP, 0, REG_DWORD, @Info^.DhcpEnabled, SizeOf(DWORD));

  DhcpEnabled := Info^.DhcpEnabled <> 0;

  if DhcpEnabled then
  begin
    Move(ZeroIp, Value, ZeroIpLen);
    Size := ZeroIpLen; 
  end else
    Size := IpListToMultiSz(@Info^.IpAddressList, Value);

  RegSetValueEx(Key, IPAddress, 0, REG_MULTI_SZ, @Value, Size);

  if DhcpEnabled then
  begin
    Move(ZeroIp, Value, ZeroIpLen);
    Size := ZeroIpLen; 
  end else
    Size := IpListToMultiSz(@Info^.IpAddressList, Value, True);

  RegSetValueEx(Key, SubnetMask, 0, REG_MULTI_SZ, @Value, Size);

  if DhcpEnabled then
  begin
    Move(ZeroIp, Value, ZeroIpLen);
    Size := ZeroIpLen; 
  end else
    Size := IpListToMultiSz(@Info^.GatewayList, Value);

  RegSetValueEx(Key, DefaultGateway, 0, REG_MULTI_SZ, @Value, Size);

  if DhcpEnabled then
  begin
    FillChar(Value, SizeOf(Value), 0);
    Size := 1;
  end else
    Size := IpListToSz(@Info^.DnsServerList, Value);

  RegSetValueEx(Key, NameServer, 0, REG_SZ, @Value, Size);

  if DhcpEnabled then
  begin
    Move(ZeroIp, Value, ZeroIpLen);
    Size := ZeroIpLen;
  end
  else
  begin
    Move(MaxIp, Value, MaxIpLen);
    Size := MaxIpLen;
  end;

  RegSetValueEx(Key, DhcpServer, 0, REG_SZ, @Value, Size);  

  if DhcpEnabled then
  begin
    Move(ZeroIp, Value, ZeroIpLen);
    Size := ZeroIpLen;

    RegSetValueEx(Key, DhcpIpAddress, 0, REG_SZ, @Value, Size);
    RegSetValueEx(Key, DhcpSubnetMask, 0, REG_SZ, @Value, Size);
    RegSetValueEx(Key, DhcpDefaultGateway, 0, REG_MULTI_SZ, @Value, Size);

    FillChar(Value, SizeOf(Value), 0);
    RegSetValueEx(Key, DhcpNameServer, 0, REG_SZ, @Value, 1);

    Size := 0;
    RegSetValueEx(Key, DhcpRetryTime, 0, REG_DWORD, @Size, SizeOf(DWORD));

    Size := 1;
    RegSetValueEx(Key, DhcpRetryStatus, 0, REG_DWORD, @Size, SizeOf(DWORD));
  end
  else
  begin
    RegDeleteValue(Key, DhcpIpAddress);
    RegDeleteValue(Key, DhcpSubnetMask);
    RegDeleteValue(Key, DhcpNameServer);
    RegDeleteValue(Key, DhcpDefaultGateway);
    RegDeleteValue(Key, DhcpDefaultGateway);
    RegDeleteValue(Key, DhcpRetryStatus);
  end;

  RegCloseKey(Key);
end;

function RipIntAdapterConfigure(Info: PRipIpConfigure): Cardinal;
var
  Sign: TMD5Digest;
begin
  if Info = nil then
  begin
    Result := ERROR_INVALID_PARAMETER;
    Exit;
  end;
  Sign := MD5Buffer(Info^.Adapter, SizeOf(TRipAdapterInfo));
  if not MD5DigestCompare(Sign, Info^.Header.Sign) then
  begin
    Result := ERROR_INVALID_PARAMETER;
    Exit;
  end;

  Result := RipIntAdapterUpdate(@Info^.Adapter);
  if Result <> ERROR_SUCCESS then
    Exit;

  Result := RipIntAdapterDisable(Info^.Adapter.AdapterName);
  if Result <> ERROR_SUCCESS then
    Exit;

  Result := RipIntAdapterEnable(Info^.Adapter.AdapterName);
end;

function RipIntGetNetworkParams(Info: PRipFixedInfo): Cardinal;
var
  FixedSize: Cardinal;
  FixedInfo: PFixedInfo;
begin
  if Info = nil then
  begin
    Result := ERROR_INVALID_PARAMETER;
    Exit;
  end;

  FixedSize := 0;
  Result := GetNetworkParams(nil, FixedSize);
  if Result <> ERROR_BUFFER_OVERFLOW then
    Exit;

  FixedInfo := AllocMem(FixedSize);
  Result := GetNetworkParams(FixedInfo, FixedSize);
  if Result = ERROR_SUCCESS then
    with Info^ do
    begin
      Move(FixedInfo^.HostName, HostName, MAX_HOSTNAME_LEN);
      Move(FixedInfo^.DomainName, DomainName, MAX_DOMAIN_NAME_LEN);
      MoveIp(@FixedInfo^.DnsServerList, @DnsServerList);
      NodeType := FixedInfo^.NodeType;
      Move(FixedInfo^.ScopeId, ScopeId, MAX_SCOPE_ID_LEN);
      EnableRouting := FixedInfo^.EnableRouting;
      EnableProxy := FixedInfo^.EnableProxy;
      EnableDns := FixedInfo^.EnableDns;
    end;

  FreeMem(FixedInfo);
end;

function RipIntGetAdaptersInfo(
  Adapter: PAdapterList; var AdapterCount: Cardinal): Cardinal;
var
  AdapterSize: Cardinal;
  AI, AdapterInfo: PIpAdapterInfo;
  MaxAdapterSize: Cardinal;
  PerAdapterSize: Cardinal;
  PerAdapterInfo: PIpPerAdapterInfo;
begin
  AdapterCount := 0;

  if Adapter = nil then
  begin
    Result := ERROR_INVALID_PARAMETER;
    Exit;
  end;

  AdapterSize := 0;
  Result := GetAdaptersInfo(nil, AdapterSize);
  if (Result <> ERROR_SUCCESS) and (Result <> ERROR_BUFFER_OVERFLOW) then
    Exit;

  if AdapterSize = 0 then
    Exit;

  AdapterInfo := AllocMem(AdapterSize);
  Result := GetAdaptersInfo(AdapterInfo, AdapterSize);
  if Result = ERROR_SUCCESS then
  begin
    AI := AdapterInfo;
    PerAdapterSize := 0;
    MaxAdapterSize := 0;
    PerAdapterInfo := nil;

    repeat
      Result := GetPerAdapterInfo(AI^.Index, nil, PerAdapterSize);
      if Result <> ERROR_BUFFER_OVERFLOW then
        Break;

      if PerAdapterSize > MaxAdapterSize then
      begin
        if PerAdapterInfo <> nil then
          FreeMem(PerAdapterInfo);
        PerAdapterInfo := AllocMem(PerAdapterSize);
        MaxAdapterSize := PerAdapterSize;
      end else
        ZeroMemory(PerAdapterInfo, MaxAdapterSize);

      Result := GetPerAdapterInfo(AI^.Index, PerAdapterInfo, PerAdapterSize);
      if Result <> ERROR_SUCCESS then
        Break;

      with Adapter^[AdapterCount] do
      begin

        Next := AI^.Next;
        ComboIndex := AI^.ComboIndex;
        Move(AI^.AdapterName, AdapterName, MAX_ADAPTER_NAME_LENGTH);
        Move(AI^.Description, Description, MAX_ADAPTER_DESCRIPTION_LENGTH);
        AddressLength := AI^.AddressLength;
        Move(AI^.Address, Address, MAX_ADAPTER_ADDRESS_LENGTH);
        Index := AI^.Index;
        Type_ := AI^.Type_;
        DhcpEnabled := AI^.DhcpEnabled;
        CurrentIpAddress := AI^.CurrentIpAddress;
        MoveIp(@AI^.IpAddressList, @IpAddressList);
        MoveIp(@AI^.GatewayList, @GatewayList);
        MoveIp(@AI^.DhcpServer, @DhcpServer);
        HaveWins := AI^.HaveWins;
        MoveIp(@AI^.PrimaryWinsServer, @PrimaryWinsServer);
        MoveIp(@AI^.SecondaryWinsServer, @SecondaryWinsServer);
        LeaseObtained := AI^.LeaseObtained;
        LeaseExpires := AI^.LeaseExpires;

        AutoconfigEnabled := PerAdapterInfo^.AutoconfigEnabled;
        AutoconfigActive := PerAdapterInfo^.AutoconfigActive;
        CurrentDnsServer := PerAdapterInfo^.CurrentDnsServer;
        MoveIp(@PerAdapterInfo^.DnsServerList, @DnsServerList);

      end;

      Inc(AdapterCount);
      AI := AI^.Next;

    until (AI = nil) or (AdapterCount > MAX_ADAPTER_COUNT);

    if PerAdapterInfo <> nil then
      FreeMem(PerAdapterInfo);
    FreeMem(AdapterInfo);
  end;

end;

{--------------------------------------------------------------------}

function RipServiceStopPending: Boolean;
begin
  Result := RipServiceStatus.dwCurrentState = SERVICE_STOP_PENDING;
end;

function RipLoadConfig(RootKey: HKEY): Boolean;
var
  RegKey: HKEY;
  cbSize: Cardinal;
begin
  { Init Defaults }
  RipPort := RipDefPort;
  RipUserPwd := MD5String('');
  RipAdminPwd := MD5String('');
  { Load From Registry }
  if RegCreateKeyEx(RootKey, RipRegKeyRipSvc, 0, nil, 0, KEY_READ, nil, RegKey, nil) = ERROR_SUCCESS then
  begin
    cbSize := SizeOf(RipPort);
    RegQueryValueEx(RegKey, RipRegValPort, nil, nil, @RipPort, @cbSize);
    cbSize := SizeOf(RipUserPwd);
    RegQueryValueEx(RegKey, RipRegValUser, nil, nil, @RipUserPwd, @cbSize);
    cbSize := SizeOf(RipAdminPwd);
    RegQueryValueEx(RegKey, RipRegValAdmin, nil, nil, @RipAdminPwd, @cbSize);
    Result := RegCloseKey(RegKey) = ERROR_SUCCESS;
  end else
    Result := True;
end;

procedure RipSetAdminPwd(RootKey: HKEY; const Passwd: string);
var
  RegKey: HKEY;
  Success: Boolean;
begin
  {$IFDEF _DEBUG}
  LogDebug('RipSetAdminPwd', []);
  {$ENDIF _DEBUG}
  
  RipAdminPwd := MD5String(Passwd);
  if RegCreateKeyEx(RootKey, PChar(RipRegKeyRipSvc),
        0, nil, 0, KEY_WRITE, nil, RegKey, nil) = ERROR_SUCCESS then
  begin
    Success := RegSetValueEx(RegKey, RipRegValAdmin, 0, REG_BINARY, @RipAdminPwd, SizeOf(RipAdminPwd)) = ERROR_SUCCESS;
    RegCloseKey(RegKey);
  end else
    Success := False;

  if not Success then
    LogError('RipSetAdminPwd, %s', [SysErrorMessage(GetLastError)]);
end;

procedure RipSetUserPwd(RootKey: HKEY; const Passwd: string);
var
  RegKey: HKEY;
  Success: Boolean;
begin
  {$IFDEF _DEBUG}
  LogDebug('RipSetUserPwd', []);
  {$ENDIF _DEBUG}
  
  RipUserPwd := MD5String(Passwd);
  if RegCreateKeyEx(RootKey, PChar(RipRegKeyRipSvc),
        0, nil, 0, KEY_WRITE, nil, RegKey, nil) = ERROR_SUCCESS then
  begin
    Success := RegSetValueEx(RegKey, RipRegValUser, 0, REG_BINARY, @RipUserPwd, SizeOf(RipUserPwd)) = ERROR_SUCCESS;
    RegCloseKey(RegKey);
  end else
    Success := False;

  if not Success then
    LogError('RipSetAdminPwd, %s', [SysErrorMessage(GetLastError)]);
end;

procedure RipSetPort(RootKey: HKEY; Port: Integer);
var
  RegKey: HKEY;
  Success: Boolean;
begin
  {$IFDEF _DEBUG}
  LogDebug('RipSetPort(%d)', [Port]);
  {$ENDIF _DEBUG}

  Success :=
    (RegCreateKeyEx(RootKey, PChar(RipRegKeyRipSvc),
        0, nil, 0, KEY_WRITE, nil, RegKey, nil) = ERROR_SUCCESS) and
    (RegSetValueEx(RegKey, PChar(RipRegValPort),
        0, REG_DWORD, @Port, SizeOf(Port)) = ERROR_SUCCESS) and
    (RegCloseKey(RegKey) = ERROR_SUCCESS);

  if not Success then
    LogError('SetPort(%d), %s', [Port, SysErrorMessage(GetLastError)]);
end;

function RipGetCommand(const Session: TSession; var Buffer;
  const Size: Cardinal; var RetSize: Cardinal): Boolean;
{$IFDEF _NET_DEBUG}
var
  s: string;
{$ENDIF _NET_DEBUG}
begin
  {$IFDEF _DEBUG}
  LogDebug('RipGetCommand', []);
  {$ENDIF _DEBUG}
  
  FillChar(Buffer, Size, 0);
  RetSize := recv(Session.Socket, Buffer, Size, 0);
  Result := Integer(RetSize) > 0;

  {$IFDEF _NET_DEBUG}
  SetString(s, PChar(@Buffer), 4);
  LogMessage('Recv: %s', [s]);
  {$ENDIF _NET_DEBUG}
end;

procedure RipTranslateCommand(var Buffer; const Size: Cardinal);
begin
  {$IFDEF _DEBUG}
  LogDebug('RipTranslateCommand', []);
  {$ENDIF _DEBUG}

  if Size = 0 then
    Exit;
  Cardinal(Buffer) := Cardinal(Buffer) and $dfdfdfdf; { UpCase }
end;

procedure RipAnsver(const Session: TSession;
  var Buffer; const Size: Cardinal);
{$IFDEF _NET_DEBUG}
var
  s: string;
{$ENDIF _NET_DEBUG}
begin
  {$IFDEF _NET_DEBUG}
  SetString(s, PChar(@Buffer), 4);
  LogMessage('Send: %s', [s]);
  {$ENDIF _NET_DEBUG}

  if send(Session.Socket, Buffer, Size, 0) = SOCKET_ERROR then
    LogError('RipAnsver, send, WSAError %d', [WSAGetLastError]);
end;

procedure RipAnsverStr(const Session: TSession;
  const Ansver: string);
begin
  RipAnsver(Session, Pointer(Ansver)^, Length(Ansver));
end;

function RipIntGetWindowsVersionString: string;
type
  OSVERSIONINFOEX = packed record
    dwOSVersionInfoSize: DWORD;
    dwMajorVersion: DWORD;
    dwMinorVersion: DWORD;
    dwBuildNumber: DWORD;
    dwPlatformId: DWORD;
    szCSDVersion: packed array[0..128-1]of Char;
    wServicePackMajor: WORD;
    wServicePackMinor: WORD;
    wSuiteMask: WORD;
    wProductType: BYTE;
    wReserved: BYTE;
  end;
const
  VER_SUITE_PERSONAL = $00000200;
  VER_NT_WORKSTATION = $0000001;
  W9x = 'Windows 9x';
  NTU = 'Windows NT Unknown';
  NT4 = 'Windows NT 4.0';
  W2K = 'Windows 2000';
  WXP = 'Windows XP';
  W2K3 = 'Windows Server 2003 family';
  WHome = 'Home Edition';
  WProf = 'Professional';
  WWork = 'Workstation';
var
  Ver: OSVERSIONINFOEX;
  Win,Typ: string;
begin
  FillChar(Ver, SizeOf(Ver), 0);
  Ver.dwOSVersionInfoSize := SizeOf(Ver);
  if GetVersionEx(POSVersionInfo(@Ver)^) then
    with Ver do
      if dwPlatformId = VER_PLATFORM_WIN32_NT then
      begin
        if dwMajorVersion = 5 then
        begin
          if dwMinorVersion = 0 then
            Win := W2K
          else if dwMinorVersion = 1 then
            Win := WXP
          else if dwMinorVersion = 2 then
            Win := W2K3
          else
            Win := NTU;
        end
          else
        if dwMajorVersion = 4 then
          Win := NT4
        else
          Win := NTU;
        if (wProductType and VER_NT_WORKSTATION) <> 0  then
        begin
          if dwMajorVersion = 5 then
          begin
            if dwMinorVersion = 0 then
              Typ := WProf
            else if dwMinorVersion = 1 then
              if (wSuiteMask and VER_SUITE_PERSONAL) <> 0 then
                Typ := WHome
              else
                Typ := WProf
          end
            else
          if dwMajorVersion = 4 then
            Typ := WWork
          else
            Typ := '';
        end;
        Result := Format('%s %s (Build: %d) %s',
          [Win, Typ, dwBuildNumber, string(szCSDVersion)]);
      end else
        Result := W9x
  else
    Result := ''
end;

procedure RipCommandVer(Session: TSession);
var
  VerStr: string;
begin
  VerStr := Format('%s %s', [RipServiceVer, RipIntGetWindowsVersionString]);
  RipAnsver(Session, Pointer(VerStr)^, Length(VerStr));
end;

function RipLogin(MD5Pwd: TMD5Digest): Integer;
begin
//  Windows.Sleep(500);
  if MD5DigestCompare(MD5Pwd, RipAdminPwd) then
    Result := RIP_ADMIN
  else if MD5DigestCompare(MD5Pwd, RipUserPwd) then
    Result := RIP_USER
  else
    Result := 0;
end;

procedure RipCommandPas(var Session: TSession;
  const Buffer; const Size: Cardinal);
var
  Passwd: string;
  MD5Pwd: TMD5Digest;
begin
  {$IFDEF _DEBUG}
  LogDebug('RipCommandPas', []);
  {$ENDIF _DEBUG}
  
  SetString(Passwd, PChar(@TByteArray(Buffer)[4]), Size-4);
  MD5Pwd := MD5String(Passwd);
  if RipLogin(MD5Pwd) <> 0 then
  begin
    Session.Passwd := MD5Pwd;
    RipAnsverStr(Session, RIPA_230); // '230-User logged in, proceed.'
  end else
    RipAnsverStr(Session, RIPA_530); // '530-Not logged in.'
end;

// list adapters (ascii-text ansver)

procedure RipCommandLsa(Session: TSession);
var
  Info: PRipNetworkInfo;
  Index: Cardinal;
  AnsverStr: string;
begin
  {$IFDEF _DEBUG}
  LogDebug('RipCommandLsa', []);
  {$ENDIF _DEBUG}

  if RipLogin(Session.Passwd) = 0 then
  begin
    RipAnsverStr(Session, RIPA_530); // '530-Not logged in.'
    Exit;
  end;

  Info := AllocMem(SizeOf(TRipNetworkInfo));

  repeat

    if RipIntGetNetworkParams(@Info^.FixedInfo) <> NO_ERROR then
    begin
      RipAnsverStr(Session, RIPA_451);
      Break;
    end;

    if RipIntGetAdaptersInfo(@Info^.Adapters, Info^.Header.AdapterCount) <> NO_ERROR then
    begin
      RipAnsverStr(Session, RIPA_451);
      Break;
    end;

    AnsverStr := ''; // RIPA_200

    with Info^.FixedInfo do
    begin
      AnsverStr := AnsverStr + Format(FixedInfoFmt, [
        string(HostName),
        string(DomainName),
        NodeTypeToStr(NodeType),
        YesNo(EnableRouting),
        YesNo(EnableProxy)]);
    end;

    for Index := 0 to Info^.Header.AdapterCount - 1 do
      with Info^.Adapters [Index] do
      begin
        AnsverStr := AnsverStr + Format(AdapterInfoFmt, [
          AdapterTypeToStr(Type_),
          string(AdapterName),
          string(Info^.FixedInfo.DomainName),
          string(Description),
          PhysAddrToStr(Address, AddressLength),
          YesNo(DhcpEnabled),
          IpListToStr(@IpAddressList),
          IpListToStr(@IpAddressList, True),
          IpListToStr(@GatewayList),
          IpListToStr(@DnsServerList)]);
      end;

    RipAnsverStr(Session, AnsverStr);
  until True;

  FreeMem(Info);
end;

// list adapters (binary ansver)

procedure RipCommandLsb(Session: TSession);
var
  Info: PRipNetworkInfo;
  Size: Cardinal;
begin
  {$IFDEF _DEBUG}
  LogDebug('RipCommandLsb', []);
  {$ENDIF _DEBUG}

  if RipLogin(Session.Passwd) = 0 then
  begin
    RipAnsverStr(Session, RIPA_530); // '530-Not logged in.'
    Exit;
  end;

  Info := AllocMem(SizeOf(TRipNetworkInfo));

  repeat

    if RipIntGetNetworkParams(@Info^.FixedInfo) <> NO_ERROR then
    begin
      RipAnsverStr(Session, RIPA_451);
      Break;
    end;

    if RipIntGetAdaptersInfo(@Info^.Adapters, Info^.Header.AdapterCount) <> NO_ERROR then
    begin
      RipAnsverStr(Session, RIPA_451);
      Break;
    end;

    with Info^.Header do
    begin
      Status := 0;//RIPC_200; 
      HeaderSize := SizeOf(TRipInfoHeader);
      FixedSize := SizeOf(TRipFixedInfo);
      AdapterSize := SizeOf(TRipAdapterInfo);
      //AdapterCount := already set
      TotalSize := HeaderSize + FixedSize + AdapterSize * AdapterCount;
      Size := FixedSize + AdapterSize * AdapterCount; 
      Sign := MD5Buffer(Info^.FixedInfo, Size);
    end;
    RipAnsver(Session, Info^, Info^.Header.TotalSize);

  until True;

  FreeMem(Info);

end;

procedure RipCommandIpc(Session: TSession;
  const Buffer; const Size: Cardinal);
var
  Access: Cardinal;
  Sign: TMD5Digest;
  Info: PRipIpConfigure;
begin
  {$IFDEF _DEBUG}
  LogDebug('RipCommandIpc', []);
  {$ENDIF _DEBUG}

  Access := RipLogin(Session.Passwd);
  if Access = 0 then
  begin
    RipAnsverStr(Session, RIPA_530); // '530-Not logged in.'
    Exit;
  end;
  if not (Access = RIP_ADMIN) then
  begin
    RipAnsverStr(Session, RIPA_550); // '550-Access denied.'
    Exit;
  end;
  if Size < SizeOf(TRipIpConfigure) then
  begin
    RipAnsverStr(Session, RIPA_501); // '501-Syntax error in parameters or arguments.'
    Exit;
  end;
  Info := @Buffer;
  if Size < Info^.Header.TotalSize then
  begin
    RipAnsverStr(Session, RIPA_501); // '501-Syntax error in parameters or arguments.'
    Exit;
  end;
  Sign := MD5Buffer(Info^.Adapter, SizeOf(TRipAdapterInfo));
  if not MD5DigestCompare(Sign, Info^.Header.Sign) then
  begin
    RipAnsverStr(Session, RIPA_501); // '501-Syntax error in parameters or arguments.'
    Exit;
  end;
  RipAnsverStr(Session, RIPA_200);

  RipIntAdapterConfigure(Info);
end;

procedure RipCommandDis(Session: TSession;
  const Buffer; const Size: Cardinal);
var
  Access: Integer;
  AdapterName: array [0..MAX_ADAPTER_NAME_LENGTH + 3] of Char;
begin
  {$IFDEF _DEBUG}
  LogDebug('RipCommandDis', []);
  {$ENDIF _DEBUG}

  Access := RipLogin(Session.Passwd);
  if Access = 0 then
  begin
    RipAnsverStr(Session, RIPA_530); // '530-Not logged in.'
    Exit;
  end;
  if not (Access = RIP_ADMIN) then
  begin
    RipAnsverStr(Session, RIPA_550); // '550-Access denied.'
    Exit;
  end;
  if (Size <= SizeOf(Cardinal)) or (Size > MAX_ADAPTER_NAME_LENGTH + 3) then
  begin
    RipAnsverStr(Session, RIPA_501); // '501-Syntax error in parameters or arguments.'
    Exit;
  end;

  FillChar(AdapterName, SizeOf(AdapterName), 0);
  StrLCopy(AdapterName, @TByteArray(Buffer)[SizeOf(Cardinal)], Size-SizeOf(Cardinal));

  RipAnsverStr(Session, RIPA_200);

  RipIntAdapterDisable(AdapterName);
end;

procedure RipCommandEna(Session: TSession;
  const Buffer; const Size: Cardinal);
var
  Access: Integer;
  AdapterName: array [0..MAX_ADAPTER_NAME_LENGTH + 3] of Char;
begin
  {$IFDEF _DEBUG}
  LogDebug('RipCommandEna', []);
  {$ENDIF _DEBUG}

  Access := RipLogin(Session.Passwd);
  if Access = 0 then
  begin
    RipAnsverStr(Session, RIPA_530); // '530-Not logged in.'
    Exit;
  end;
  if not (Access = RIP_ADMIN) then
  begin
    RipAnsverStr(Session, RIPA_550); // '550-Access denied.'
    Exit;
  end;
  if (Size <= SizeOf(Cardinal)) or (Size > MAX_ADAPTER_NAME_LENGTH + 3) then
  begin
    RipAnsverStr(Session, RIPA_501); // '501-Syntax error in parameters or arguments.'
    Exit;
  end;

  FillChar(AdapterName, SizeOf(AdapterName), 0);
  StrLCopy(AdapterName, @TByteArray(Buffer)[SizeOf(Cardinal)], Size-SizeOf(Cardinal));

  RipAnsverStr(Session, RIPA_200);

  RipIntAdapterEnable(AdapterName);
end;

procedure RipCommandSdw(Session: TSession);
var
  Access: Integer;
begin
  {$IFDEF _DEBUG}
  LogDebug('RipCommandSdw', []);
  {$ENDIF _DEBUG}

  Access := RipLogin(Session.Passwd);
  if Access = 0 then
  begin
    RipAnsverStr(Session, RIPA_530); // '530-Not logged in.'
    Exit;
  end;
  if not (Access = RIP_ADMIN) then
  begin
    RipAnsverStr(Session, RIPA_550); // '550-Access denied.'
    Exit;
  end;

  if RipIntSystemShutdown('', 0, FALSE) then
    RipAnsverStr(Session, RIPA_200)
  else
    RipAnsverStr(Session, RIPA_451+' '+SysErrorMessage(GetLastError));
end;

procedure RipCommandRsw(Session: TSession);
var
  Access: Integer;
begin
  {$IFDEF _DEBUG}
  LogDebug('RipCommandRsw', []);
  {$ENDIF _DEBUG}

  Access := RipLogin(Session.Passwd);
  if Access = 0 then
  begin
    RipAnsverStr(Session, RIPA_530); // '530-Not logged in.'
    Exit;
  end;
  if not (Access = RIP_ADMIN) then
  begin
    RipAnsverStr(Session, RIPA_550); // '550-Access denied.'
    Exit;
  end;

  if RipIntSystemShutdown('', 0, TRUE) then
    RipAnsverStr(Session, RIPA_200)
  else
    RipAnsverStr(Session, RIPA_451+' '+SysErrorMessage(GetLastError));
end;

procedure RipCoomandQuit(var Session: TSession);
begin
  RipAnsverStr(Session, RIPA_221); // '221-Service closing control connection.'
  Session.Active := False;
end;

procedure RipCommandUnknown(Session: TSession);
begin
  RipAnsverStr(Session, RIPA_500); // '500-Syntax error, command unrecognized.'
end;

procedure RipDispatchCommand(var Session: TSession;
  const Buffer; const Size: Cardinal);
begin
  {$IFDEF _DEBUG}
  LogDebug('RipDispatchCommand', []);
  {$ENDIF _DEBUG}

  if Size = 0 then
    Exit;
  case Cardinal(Buffer) of
  RIPC_VER: RipCommandVer(Session);
  RIPC_PAS: RipCommandPas(Session, Buffer, Size);
  RIPC_LSA: RipCommandLsa(Session);
  RIPC_LSB: RipCommandLsb(Session);
  RIPC_IPC: RipCommandIpc(Session, Buffer, Size);
  RIPC_DIS: RipCommandDis(Session, Buffer, Size);
  RIPC_ENA: RipCommandEna(Session, Buffer, Size);
  RIPC_SDW: RipCommandSdw(Session);
  RIPC_RSW: RipCommandRsw(Session);
  RIPC_QUIT: RipCoomandQuit(Session);
  else RipCommandUnknown(Session);
  end;
end;

function RipThreadProc(Param: Pointer): Cardinal; stdcall;
var
  LocalSoket: TSOCKET;
  LocalSockAddr: TSockAddr;
  SizeOfSockAddr: Integer;
  Buffer: array[0..4098]of Char;
  BufLen: Cardinal;
  Session: TSession;
begin
  {$IFDEF _DEBUG}
  LogDebug('RipThreadProc', []);
  {$ENDIF _DEBUG}

  while True do
  begin
    FillChar(LocalSockAddr, SizeOf(LocalSockAddr), 0);
    SizeOfSockAddr := SizeOf(LocalSockAddr);
    LocalSoket := accept(RipWSASocket, @LocalSockAddr, @SizeOfSockAddr);
    if LocalSoket = SOCKET_ERROR then
    begin
      LogError('RipThreadProc, accept, WSAError %d', [WSAGetLastError]);
      Break;
    end;

    {$IFDEF _NET_DEBUG}
    with TIPAddr(LocalSockAddr.sin_addr.S_addr) do
      LogMessage('Connection accepted from %d.%d.%d.%d', [s_b1, s_b2, s_b3, s_b4]);
    {$ENDIF _NET_DEBUG}

    Session.Active := True;
    Session.Socket := LocalSoket;
    Session.SockAddr := LocalSockAddr;
    Session.Passwd := MD5String('');

    RipAnsverStr(Session, RIPA_220); // '220-Service ready for new user.'
    
    while Session.Active and
          RipGetCommand(Session, Buffer, SizeOf(Buffer), BufLen) do
    begin
      RipTranslateCommand(Buffer, BufLen);
      RipDispatchCommand(Session, Buffer, BufLen);
    end;

    if not (shutdown(LocalSoket, 0) = ERROR_SUCCESS) then
      LogError('RipThreadProc, shutdown, WSAError %d', [WSAGetLastError]);

    {$IFDEF _NET_DEBUG}
    LogMessage('Connection closed', []);
    {$ENDIF _NET_DEBUG}
  end;
  Result := 0;
end;

function RipServiceInitialization(argc : DWord; var argv : array of PChar): BOOL;
begin
  {$IFDEF _DEBUG}
  LogDebug('RipServiceInitialization', []);
  {$ENDIF _DEBUG}

  if not RipLoadConfig(HKEY_LOCAL_MACHINE) then
  begin
    LogError('RipLoadConfig, %s', [SysErrorMessage(GetLastError)]);
    Result := False;
    Exit;
  end;

  if not (WSAStartup(2, RipWSAData) = ERROR_SUCCESS) then
  begin
    LogError('WSAStartup, WSAError %d', [WSAGetLastError]);
    Result := False;
    Exit;
  end;

  RipWSASocket := socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
  if RipWSASocket = INVALID_SOCKET then
  begin
    LogError('socket, WSAError %d', [WSAGetLastError]);
    Result := False;
    Exit;
  end;

  FillChar(RipSockAddr, SizeOf(RipSockAddr), 0);
  RipSockAddr.sa_family := AF_INET;
  RipSockAddr.sin_port := htons(u_short(RipPort));
  
  if not (bind(RipWSASocket, RipSockAddr, SizeOf(RipSockAddr)) = ERROR_SUCCESS) then
  begin
    LogError('bind, WSAError %d', [WSAGetLastError]);
    Result := False;
    Exit;
  end;

  if not (listen(RipWSASocket, 1) = ERROR_SUCCESS) then
  begin
    LogError('listen, WSAError %d', [WSAGetLastError]);
    Result := False;
    Exit;
  end;
            
  RipThread := CreateThread(
    nil,
    0,
    @RipThreadProc,
    Pointer(RipWSASocket),
    0,
    RipThreadId);

  Result := True;
end;

function RipServiceShutdown(): Boolean;
begin
  {$IFDEF _DEBUG}
  LogDebug('RipServiceShutdown', []);
  {$ENDIF _DEBUG}

  if WaitForSingleObject(RipThread, 50) = WAIT_TIMEOUT then
    if not TerminateThread(RipThread, 0) then
    begin
      LogError('RipServiceShutdown, TerminateThread, %s',
        [SysErrorMessage(GetLastError)]);
    end;

//  if not shutdown(RipWSASocket, 0) = ERROR_SUCCESS then
//    LogError('RipServiceShutdown, shutdown, WSAError %d', [WSAGetLastError]);

  shutdown(RipWSASocket, 0); 

  Result := True;
end;

function RipSetServiceStatus(Status: Cardinal): Boolean;
begin
  {$IFDEF _DEBUG}
  LogDebug('RipSetServiceStatus(%d)', [Status]);
  {$ENDIF _DEBUG}

  RipServiceStatus.dwServiceType      := SERVICE_WIN32;
  RipServiceStatus.dwCurrentState     := Status;
  RipServiceStatus.dwControlsAccepted := SERVICE_ACCEPT_STOP;
  RipServiceStatus.dwWin32ExitCode    := 0;
  RipServiceStatus.dwServiceSpecificExitCode := 0;
  RipServiceStatus.dwCheckPoint       := 0;
  RipServiceStatus.dwWaitHint         := 0;

  Result := False;
  if RipStatusHandle <> 0 then
  begin
    Result := SetServiceStatus(RipStatusHandle, RipServiceStatus);
    if not Result then LogError('RipSetServiceStatus(%d) %s',
      [Status, SysErrorMessage(GetLastError())]);
  end else
    LogError('RipSetServiceStatus, Invalid StatusHandle', []);
end;

procedure RipServiceCtrlHandler(Opcode : Cardinal);stdcall;
begin
  {$IFDEF _DEBUG}
  LogDebug('RipServiceCtrlHandler(%d)', [Opcode]);
  {$ENDIF _DEBUG}
  
  case Opcode of
    SERVICE_CONTROL_STOP:
    begin
      RipServiceStatus.dwWin32ExitCode := 0;
      RipServiceStatus.dwCurrentState  := SERVICE_STOP_PENDING;
      RipServiceStatus.dwCheckPoint    := 0;
      RipServiceStatus.dwWaitHint      := 0;
    end;
  end;

  if not SetServiceStatus(RipStatusHandle, RipServiceStatus) then
    LogError('RipServiceCtrlHandler, SetServiceStatus, %s', [SysErrorMessage(GetLastError)]);
end;

procedure RipServiceProc(argc : DWord; var argv : array of PChar); stdcall;
begin
  {$IFDEF _DEBUG}
  LogDebug('RipServiceProc', []);
  {$ENDIF _DEBUG}

  RipServiceStatus.dwServiceType      := SERVICE_WIN32;
  RipServiceStatus.dwCurrentState     := SERVICE_START_PENDING;
  RipServiceStatus.dwControlsAccepted := SERVICE_ACCEPT_STOP;
  RipServiceStatus.dwWin32ExitCode    := 0;
  RipServiceStatus.dwServiceSpecificExitCode := 0;
  RipServiceStatus.dwCheckPoint       := 0;
  RipServiceStatus.dwWaitHint         := 0;

  RipStatusHandle := RegisterServiceCtrlHandler(RipServiceName, @RipServiceCtrlHandler);
  if RipStatusHandle = 0 then
    LogError('RegisterServiceCtrlHandler, %s', [SysErrorMessage(GetLastError)]);

  if not RipSetServiceStatus(SERVICE_START_PENDING) then
    Exit;    

  if not RipServiceInitialization(argc, argv) then
  begin
    LogError('RipServiceInitialization, %s', [SysErrorMessage(GetLastError)]);
    RipSetServiceStatus(SERVICE_STOPPED);
    Exit;
  end;

  if not RipSetServiceStatus(SERVICE_RUNNING) then
    Exit;

  while (WaitForSingleObject(RipThread, 50) = WAIT_TIMEOUT) and
        (RipServiceStatus.dwCurrentState = SERVICE_RUNNING) do
    ;

  if not RipServiceShutdown() then
    LogError('RipServiceShutdown, %s', [SysErrorMessage(GetLastError)]);

  RipSetServiceStatus(SERVICE_STOPPED);
end;

procedure RipStartServiceCtrlDispatcher;
var
  DispatchTable: packed array[0..1]of _SERVICE_TABLE_ENTRYA;
begin
  {$IFDEF _DEBUG}
  LogDebug('RipStartService', []);
  {$ENDIF _DEBUG}

  {$IFDEF _USE_RTL_CRITICAL_SECTION}
  FillChar(RipLogSection, SizeOf(RipLogSection), 0);
  InitializeCriticalSection(RipLogSection);
  {$ENDIF _USE_RTL_CRITICAL_SECTION}

  DispatchTable[0].lpServiceName := RipServiceName;
  DispatchTable[0].lpServiceProc := @RipServiceProc;
  DispatchTable[1].lpServiceName := nil;
  DispatchTable[1].lpServiceProc := nil;

  if not StartServiceCtrlDispatcher(DispatchTable[0]) then
    LogError('StartServiceCtrlDispatcher, %s', [SysErrorMessage(GetLastError)]);

  {$IFDEF _USE_RTL_CRITICAL_SECTION}
  DeleteCriticalSection(RipLogSection);
  {$ENDIF _USE_RTL_CRITICAL_SECTION}
end;

procedure RipInstallService;
var
  Manager: SC_HANDLE;
  Service: SC_HANDLE;
  ShortPath: array[0..MAX_PATH]of Char;
begin
  {$IFDEF _DEBUG}
  LogDebug('RipInsatllService', []);
  {$ENDIF _DEBUG}
  
  { Install }
  Manager := OpenSCManager(nil, SERVICES_ACTIVE_DATABASE, SC_MANAGER_ALL_ACCESS);
  if Manager = 0 then
  begin
    LogError('OpenSCManager, %s', [SysErrorMessage(GetLastError)]);
    Exit;
  end;

  FillChar(ShortPath, SizeOf(ShortPath), 0);
  if GetShortPathName(PChar(ParamStr(0)), ShortPath, MAX_PATH) = 0 then
  begin
    LogError('GetShortPathName, %s', [SysErrorMessage(GetLastError)]);
    Exit;
  end;
  StrCat(ShortPath, ' ');
  StrCat(ShortPath, param_ntservice);

  Service := CreateService(Manager,
    RipServiceName,
    RipServiceTitle,
    SC_MANAGER_ALL_ACCESS,
    SERVICE_WIN32_OWN_PROCESS,
    SERVICE_AUTO_START,
    SERVICE_ERROR_NORMAL,
    ShortPath,
    nil, // no group
    nil, // no tag
    nil, // no load ordering
    nil, // LocalSystem account
    nil);// No Password
    
  if Service <> 0 then
    CloseServiceHandle(Service)
  else
    LogError('CreateService, %s', [SysErrorMessage(GetLastError)]);

  CloseServiceHandle(Manager);
end;

procedure RipStartService;
var
  Manager: SC_HANDLE;
  Service: SC_HANDLE;
  argc: Cardinal;
  argv: PAnsiChar;
begin
  {$IFDEF _DEBUG}
  LogDebug('RipStartService', []);
  {$ENDIF _DEBUG}
  
  Manager := OpenSCManager(nil, SERVICES_ACTIVE_DATABASE, SC_MANAGER_ALL_ACCESS);
  if Manager = 0 then
  begin
    LogError('OpenSCManager, %s', [SysErrorMessage(GetLastError)]);
    Exit;
  end;
  Service := OpenService(Manager, RipServiceName, SC_MANAGER_ALL_ACCESS);
  if Service <> 0 then
  begin
    argc := 0;
    argv := nil;
    if not StartService(Service, argc, argv) then
      LogError('StartService, %s', [SysErrorMessage(GetLastError)]);
    CloseServiceHandle(Service);
  end else
    LogError('OpenService, %s', [SysErrorMessage(GetLastError)]);
  CloseServiceHandle(Manager);
end;

procedure RipStopService;
var
  Manager: SC_HANDLE;
  Service: SC_HANDLE;
  Status: _SERVICE_STATUS;
begin
  {$IFDEF _DEBUG}
  LogDebug('RipStopService', []);
  {$ENDIF _DEBUG}

  Manager := OpenSCManager(nil, SERVICES_ACTIVE_DATABASE, SC_MANAGER_ALL_ACCESS);
  if Manager = 0 then
  begin
    LogError('OpenSCManager, %s', [SysErrorMessage(GetLastError)]);
    Exit;
  end;
  Service := OpenService(Manager, RipServiceName, SC_MANAGER_ALL_ACCESS);
  if Service <> 0 then
  begin
    if not ControlService(Service, SERVICE_CONTROL_STOP, Status) then
      LogError('ControlService(%d), %s', [SERVICE_CONTROL_STOP, SysErrorMessage(GetLastError)]);
    CloseServiceHandle(Service);
  end;
  CloseServiceHandle(Manager);
end;

procedure RipUninstallService;
var
  Manager: SC_HANDLE;
  Service: SC_HANDLE;
begin
  {$IFDEF _DEBUG}
  LogDebug('RipUninstallService', []);
  {$ENDIF _DEBUG}

  Manager := OpenSCManager(nil, SERVICES_ACTIVE_DATABASE, SC_MANAGER_ALL_ACCESS);
  if Manager = 0 then
  begin
    LogError('OpenSCManager, %s', [SysErrorMessage(GetLastError)]);
    Exit;
  end;
  Service := OpenService(Manager, RipServiceName, SC_MANAGER_ALL_ACCESS);
  if Service <> 0 then
  begin
    if not DeleteService(Service) then
      LogError('DeleteService, %s', [SysErrorMessage(GetLastError)]);
    CloseServiceHandle(Service);
  end else
    LogError('OpenService, %s', [SysErrorMessage(GetLastError)]);
  CloseServiceHandle(Manager)
end;

{**********************************************************

usage: ripntsvc [options]

  -i --install
  -r --uninstall
  -s --start
  -t --stop

  -a --admin [password]
  -u --user [password]
  -p --port [port]

     --ntserive

**********************************************************}

//
// function RipParseParams: Boolean;
// return True if --ntserive present
//

function RipParseParams: Boolean;
var
  Index: Integer;
  Param: string;
  Port,Code: Integer;
begin
  {$IFDEF _DEBUG}
  LogDebug('RipParseParams', []);
  {$ENDIF _DEBUG}
  
  Result := False;
  Index := 1;
  while Index <= ParamCount do
  begin
    Param := ParamStr(Index);
    Inc(Index);
    if SameText(Param, param_ntservice) then
      Result := True
    else
    if SameText(Param, param_s) or
       SameText(Param, param_start) then
      RipStartService
    else
    if SameText(Param, param_t) or
       SameText(Param, param_stop) then
      RipStopService
    else
    if SameText(Param, param_a) or
       SameText(Param, param_admin) then
    begin
      Param := ParamStr(Index);
      if (Length(Param) > 0) and (Param[1] <> '-') then
        Inc(Index) else Param := '';
      RipSetAdminPwd(HKEY_LOCAL_MACHINE, Param);
    end else
    if SameText(Param, param_u) or
       SameText(Param, param_user) then
    begin
      Param := ParamStr(Index);
      if (Length(Param) > 0) and (Param[1] <> '-') then
        Inc(Index) else Param := '';
      RipSetUserPwd(HKEY_LOCAL_MACHINE, Param);
    end else
    if SameText(Param, param_p) or
       SameText(Param, param_port) then
    begin
      Param := ParamStr(Index);
      Val(Param, Port, Code);
      if (Code = 0) and (Port > 0) and (Port < MaxWord) then
        Inc(Index) else Port := RipDefPort;
      RipSetPort(HKEY_LOCAL_MACHINE, Port);
    end else
    if SameText(Param, param_i) or
       SameText(Param, param_install) then
      RipInstallService
    else
    if SameText(Param, param_r) or
       SameText(Param, param_uninstall) then
      RipUninstallService
  end;
end;

begin
  {$IFDEF _DEBUG}
  LogDebug('----------------------------------------', []);
  LogDebug(ParamStr(0), []);
  {$ENDIF _DEBUG}

  if RipParseParams then
    RipStartServiceCtrlDispatcher;
end.
